# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: SimulatorTables.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'SimulatorTables.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15SimulatorTables.proto\x12\x04rips\"?\n\x15SimulatorTableRequest\x12\x15\n\rwellpath_name\x18\x01 \x01(\t\x12\x0f\n\x07\x63\x61se_id\x18\x02 \x01(\x05\"G\n\x1cSimulatorTableUnifiedRequest\x12\x16\n\x0ewellpath_names\x18\x01 \x03(\t\x12\x0f\n\x07\x63\x61se_id\x18\x02 \x01(\x05\"\xd2\x04\n\x15SimulatorCompdatEntry\x12\x11\n\twell_name\x18\x01 \x01(\t\x12\x0e\n\x06grid_i\x18\x02 \x01(\x05\x12\x0e\n\x06grid_j\x18\x03 \x01(\x05\x12\x0f\n\x07upper_k\x18\x04 \x01(\x05\x12\x0f\n\x07lower_k\x18\x05 \x01(\x05\x12\x16\n\x0eopen_shut_flag\x18\x06 \x01(\t\x12\x17\n\nsaturation\x18\x07 \x01(\x01H\x00\x88\x01\x01\x12\x1d\n\x10transmissibility\x18\x08 \x01(\x01H\x01\x88\x01\x01\x12\x15\n\x08\x64iameter\x18\t \x01(\x01H\x02\x88\x01\x01\x12\x0f\n\x02kh\x18\n \x01(\x01H\x03\x88\x01\x01\x12\x18\n\x0bskin_factor\x18\x0b \x01(\x01H\x04\x88\x01\x01\x12\x15\n\x08\x64_factor\x18\x0c \x01(\x01H\x05\x88\x01\x01\x12\x16\n\tdirection\x18\r \x01(\tH\x06\x88\x01\x01\x12\x15\n\x08start_md\x18\x0e \x01(\x01H\x07\x88\x01\x01\x12\x13\n\x06\x65nd_md\x18\x0f \x01(\x01H\x08\x88\x01\x01\x12\x14\n\x07\x63omment\x18\x10 \x01(\tH\t\x88\x01\x01\x12\x16\n\tgrid_name\x18\x11 \x01(\tH\n\x88\x01\x01\x12\x1e\n\x11\x63ompletion_number\x18\x12 \x01(\x05H\x0b\x88\x01\x01\x42\r\n\x0b_saturationB\x13\n\x11_transmissibilityB\x0b\n\t_diameterB\x05\n\x03_khB\x0e\n\x0c_skin_factorB\x0b\n\t_d_factorB\x0c\n\n_directionB\x0b\n\t_start_mdB\t\n\x07_end_mdB\n\n\x08_commentB\x0c\n\n_grid_nameB\x14\n\x12_completion_number\"\x80\x04\n\x16SimulatorWelspecsEntry\x12\x11\n\twell_name\x18\x01 \x01(\t\x12\x12\n\ngroup_name\x18\x02 \x01(\t\x12\x0e\n\x06grid_i\x18\x03 \x01(\x05\x12\x0e\n\x06grid_j\x18\x04 \x01(\x05\x12\x16\n\tbhp_depth\x18\x05 \x01(\x01H\x00\x88\x01\x01\x12\r\n\x05phase\x18\x06 \x01(\t\x12\x1c\n\x0f\x64rainage_radius\x18\x07 \x01(\x01H\x01\x88\x01\x01\x12\x1c\n\x0finflow_equation\x18\x08 \x01(\tH\x02\x88\x01\x01\x12\x19\n\x0c\x61uto_shut_in\x18\t \x01(\tH\x03\x88\x01\x01\x12\x17\n\ncross_flow\x18\n \x01(\tH\x04\x88\x01\x01\x12\x14\n\x07pvt_num\x18\x0b \x01(\x05H\x05\x88\x01\x01\x12%\n\x18hydrostatic_density_calc\x18\x0c \x01(\tH\x06\x88\x01\x01\x12\x17\n\nfip_region\x18\r \x01(\x05H\x07\x88\x01\x01\x12\x16\n\tgrid_name\x18\x0e \x01(\tH\x08\x88\x01\x01\x42\x0c\n\n_bhp_depthB\x12\n\x10_drainage_radiusB\x12\n\x10_inflow_equationB\x0f\n\r_auto_shut_inB\r\n\x0b_cross_flowB\n\n\x08_pvt_numB\x1b\n\x19_hydrostatic_density_calcB\r\n\x0b_fip_regionB\x0c\n\n_grid_name\"\xfe\x01\n\x1bSimulatorWelsegsHeaderEntry\x12\x11\n\twell_name\x18\x01 \x01(\t\x12\x11\n\ttop_depth\x18\x02 \x01(\x01\x12\x12\n\ntop_length\x18\x03 \x01(\x01\x12\x1c\n\x0fwellbore_volume\x18\x04 \x01(\x01H\x00\x88\x01\x01\x12\x11\n\tinfo_type\x18\x05 \x01(\t\x12 \n\x13pressure_components\x18\x06 \x01(\tH\x01\x88\x01\x01\x12\x17\n\nflow_model\x18\x07 \x01(\tH\x02\x88\x01\x01\x42\x12\n\x10_wellbore_volumeB\x16\n\x14_pressure_componentsB\r\n\x0b_flow_model\"\xf9\x01\n\x18SimulatorWelsegsRowEntry\x12\x11\n\tsegment_1\x18\x01 \x01(\x05\x12\x11\n\tsegment_2\x18\x02 \x01(\x05\x12\x0e\n\x06\x62ranch\x18\x03 \x01(\x05\x12\x14\n\x0cjoin_segment\x18\x04 \x01(\x05\x12\x0e\n\x06length\x18\x05 \x01(\x01\x12\r\n\x05\x64\x65pth\x18\x06 \x01(\x01\x12\x15\n\x08\x64iameter\x18\x07 \x01(\x01H\x00\x88\x01\x01\x12\x16\n\troughness\x18\x08 \x01(\x01H\x01\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\t \x01(\tH\x02\x88\x01\x01\x42\x0b\n\t_diameterB\x0c\n\n_roughnessB\x0e\n\x0c_description\"w\n\x15SimulatorWelsegsEntry\x12\x31\n\x06header\x18\x01 \x01(\x0b\x32!.rips.SimulatorWelsegsHeaderEntry\x12+\n\x03row\x18\x02 \x03(\x0b\x32\x1e.rips.SimulatorWelsegsRowEntry\"\x9d\x01\n\x16SimulatorCompsegsEntry\x12\t\n\x01i\x18\x01 \x01(\x05\x12\t\n\x01j\x18\x02 \x01(\x05\x12\t\n\x01k\x18\x03 \x01(\x05\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x05\x12\x16\n\x0e\x64istance_start\x18\x05 \x01(\x01\x12\x14\n\x0c\x64istance_end\x18\x06 \x01(\x01\x12\x16\n\tgrid_name\x18\x07 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_grid_name\"\xad\x02\n\x16SimulatorWsegvalvEntry\x12\x11\n\twell_name\x18\x01 \x01(\t\x12\x16\n\x0esegment_number\x18\x02 \x01(\x05\x12\n\n\x02\x63v\x18\x03 \x01(\x01\x12\x0c\n\x04\x61rea\x18\x04 \x01(\x01\x12\x19\n\x0c\x65xtra_length\x18\x05 \x01(\x01H\x00\x88\x01\x01\x12\x13\n\x06pipe_d\x18\x06 \x01(\x01H\x01\x88\x01\x01\x12\x16\n\troughness\x18\x07 \x01(\x01H\x02\x88\x01\x01\x12\x13\n\x06pipe_a\x18\x08 \x01(\x01H\x03\x88\x01\x01\x12\x13\n\x06status\x18\t \x01(\tH\x04\x88\x01\x01\x12\x12\n\x05max_a\x18\n \x01(\x01H\x05\x88\x01\x01\x42\x0f\n\r_extra_lengthB\t\n\x07_pipe_dB\x0c\n\n_roughnessB\t\n\x07_pipe_aB\t\n\x07_statusB\x08\n\x06_max_a\"\x86\x07\n\x16SimulatorWsegaicdEntry\x12\x11\n\twell_name\x18\x01 \x01(\t\x12\x11\n\tsegment_1\x18\x02 \x01(\x05\x12\x11\n\tsegment_2\x18\x03 \x01(\x05\x12\x10\n\x08strength\x18\x04 \x01(\x01\x12\x13\n\x06length\x18\x05 \x01(\x01H\x00\x88\x01\x01\x12\x19\n\x0c\x64\x65nsity_cali\x18\x06 \x01(\x01H\x01\x88\x01\x01\x12\x1b\n\x0eviscosity_cali\x18\x07 \x01(\x01H\x02\x88\x01\x01\x12\x1b\n\x0e\x63ritical_value\x18\x08 \x01(\x01H\x03\x88\x01\x01\x12\x18\n\x0bwidth_trans\x18\t \x01(\x01H\x04\x88\x01\x01\x12\x1b\n\x0emax_visc_ratio\x18\n \x01(\x01H\x05\x88\x01\x01\x12\"\n\x15method_scaling_factor\x18\x0b \x01(\x05H\x06\x88\x01\x01\x12\x14\n\x0cmax_abs_rate\x18\x0c \x01(\x01\x12\x1a\n\x12\x66low_rate_exponent\x18\r \x01(\x01\x12\x15\n\rvisc_exponent\x18\x0e \x01(\x01\x12\x13\n\x06status\x18\x0f \x01(\tH\x07\x88\x01\x01\x12\x1e\n\x11oil_flow_fraction\x18\x10 \x01(\x01H\x08\x88\x01\x01\x12 \n\x13water_flow_fraction\x18\x11 \x01(\x01H\t\x88\x01\x01\x12\x1e\n\x11gas_flow_fraction\x18\x12 \x01(\x01H\n\x88\x01\x01\x12\x1e\n\x11oil_visc_fraction\x18\x13 \x01(\x01H\x0b\x88\x01\x01\x12 \n\x13water_visc_fraction\x18\x14 \x01(\x01H\x0c\x88\x01\x01\x12\x1e\n\x11gas_visc_fraction\x18\x15 \x01(\x01H\r\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x16 \x01(\tH\x0e\x88\x01\x01\x42\t\n\x07_lengthB\x0f\n\r_density_caliB\x11\n\x0f_viscosity_caliB\x11\n\x0f_critical_valueB\x0e\n\x0c_width_transB\x11\n\x0f_max_visc_ratioB\x18\n\x16_method_scaling_factorB\t\n\x07_statusB\x14\n\x12_oil_flow_fractionB\x16\n\x14_water_flow_fractionB\x14\n\x12_gas_flow_fractionB\x14\n\x12_oil_visc_fractionB\x16\n\x14_water_visc_fractionB\x14\n\x12_gas_visc_fractionB\x0e\n\x0c_description\"\x81\x01\n\x15SimulatorWpimultEntry\x12\x11\n\twell_name\x18\x01 \x01(\t\x12\x0e\n\x06pimult\x18\x02 \x01(\x01\x12\t\n\x01i\x18\x03 \x01(\x05\x12\t\n\x01j\x18\x04 \x01(\x05\x12\t\n\x01k\x18\x05 \x01(\x05\x12\x16\n\tgrid_name\x18\x06 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_grid_name\"\xde\x02\n\x12SimulatorTableData\x12,\n\x07\x63ompdat\x18\x01 \x03(\x0b\x32\x1b.rips.SimulatorCompdatEntry\x12.\n\x08welspecs\x18\x02 \x03(\x0b\x32\x1c.rips.SimulatorWelspecsEntry\x12,\n\x07welsegs\x18\x03 \x03(\x0b\x32\x1b.rips.SimulatorWelsegsEntry\x12.\n\x08\x63ompsegs\x18\x04 \x03(\x0b\x32\x1c.rips.SimulatorCompsegsEntry\x12.\n\x08wsegvalv\x18\x05 \x03(\x0b\x32\x1c.rips.SimulatorWsegvalvEntry\x12.\n\x08wsegaicd\x18\x06 \x03(\x0b\x32\x1c.rips.SimulatorWsegaicdEntry\x12,\n\x07wpimult\x18\x07 \x03(\x0b\x32\x1b.rips.SimulatorWpimultEntry2\xb4\x01\n\x08WellPath\x12L\n\x11GetCompletionData\x12\x1b.rips.SimulatorTableRequest\x1a\x18.rips.SimulatorTableData\"\x00\x12Z\n\x18GetCompletionDataUnified\x12\".rips.SimulatorTableUnifiedRequest\x1a\x18.rips.SimulatorTableData\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'SimulatorTables_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SIMULATORTABLEREQUEST']._serialized_start=31
  _globals['_SIMULATORTABLEREQUEST']._serialized_end=94
  _globals['_SIMULATORTABLEUNIFIEDREQUEST']._serialized_start=96
  _globals['_SIMULATORTABLEUNIFIEDREQUEST']._serialized_end=167
  _globals['_SIMULATORCOMPDATENTRY']._serialized_start=170
  _globals['_SIMULATORCOMPDATENTRY']._serialized_end=764
  _globals['_SIMULATORWELSPECSENTRY']._serialized_start=767
  _globals['_SIMULATORWELSPECSENTRY']._serialized_end=1279
  _globals['_SIMULATORWELSEGSHEADERENTRY']._serialized_start=1282
  _globals['_SIMULATORWELSEGSHEADERENTRY']._serialized_end=1536
  _globals['_SIMULATORWELSEGSROWENTRY']._serialized_start=1539
  _globals['_SIMULATORWELSEGSROWENTRY']._serialized_end=1788
  _globals['_SIMULATORWELSEGSENTRY']._serialized_start=1790
  _globals['_SIMULATORWELSEGSENTRY']._serialized_end=1909
  _globals['_SIMULATORCOMPSEGSENTRY']._serialized_start=1912
  _globals['_SIMULATORCOMPSEGSENTRY']._serialized_end=2069
  _globals['_SIMULATORWSEGVALVENTRY']._serialized_start=2072
  _globals['_SIMULATORWSEGVALVENTRY']._serialized_end=2373
  _globals['_SIMULATORWSEGAICDENTRY']._serialized_start=2376
  _globals['_SIMULATORWSEGAICDENTRY']._serialized_end=3278
  _globals['_SIMULATORWPIMULTENTRY']._serialized_start=3281
  _globals['_SIMULATORWPIMULTENTRY']._serialized_end=3410
  _globals['_SIMULATORTABLEDATA']._serialized_start=3413
  _globals['_SIMULATORTABLEDATA']._serialized_end=3763
  _globals['_WELLPATH']._serialized_start=3766
  _globals['_WELLPATH']._serialized_end=3946
# @@protoc_insertion_point(module_scope)



import uuid

from .pdmobject import add_method
from .project import Project
from .resinsight_classes import RegularSurface
from typing import List


@add_method(RegularSurface)
def set_property(self: RegularSurface, name: str, values: List[float]) -> None:
    """Sets a property on a regular surface.

    Arguments:
        name(str): Name of the property.
        values (List[float]): Values to set (float32).
          Should be of nx*ny size (see RegularSurface).
    """

    key = "{}_{}".format(uuid.uuid4(), "regular_surface_key")

    project = self.ancestor(Project)
    if project:
        project.set_key_values(key, values)
        self.set_property_from_key(name=name, value_key=key)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: KeyValueStore.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'KeyValueStore.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Definitions_pb2 as Definitions__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13KeyValueStore.proto\x12\x04rips\x1a\x11\x44\x65\x66initions.proto\"\x94\x01\n\x17KeyValueStoreInputChunk\x12\x33\n\nparameters\x18\x01 \x01(\x0b\x32\x1d.rips.KeyValueInputParametersH\x00\x12*\n\x06values\x18\x02 \x01(\x0b\x32\x18.rips.KeyValueStoreChunkH\x00\x42\x18\n\x16KeyValueStoreChunkType\"=\n\x17KeyValueInputParameters\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0cnum_elements\x18\x02 \x01(\x03\"$\n\x12KeyValueStoreChunk\x12\x0e\n\x06values\x18\x01 \x03(\x02\"*\n\x1aKeyValueStoreOutputRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"*\n\x18KeyValueStoreOutputChunk\x12\x0e\n\x06values\x18\x01 \x03(\x02\"*\n\x1aKeyValueStoreRemoveRequest\x12\x0c\n\x04name\x18\x01 \x01(\t2\xf1\x01\n\rKeyValueStore\x12N\n\x08SetValue\x12\x1d.rips.KeyValueStoreInputChunk\x1a\x1f.rips.ClientToServerStreamReply\"\x00(\x01\x12P\n\x08GetValue\x12 .rips.KeyValueStoreOutputRequest\x1a\x1e.rips.KeyValueStoreOutputChunk\"\x00\x30\x01\x12>\n\x0bRemoveValue\x12 .rips.KeyValueStoreRemoveRequest\x1a\x0b.rips.Empty\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'KeyValueStore_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_KEYVALUESTOREINPUTCHUNK']._serialized_start=49
  _globals['_KEYVALUESTOREINPUTCHUNK']._serialized_end=197
  _globals['_KEYVALUEINPUTPARAMETERS']._serialized_start=199
  _globals['_KEYVALUEINPUTPARAMETERS']._serialized_end=260
  _globals['_KEYVALUESTORECHUNK']._serialized_start=262
  _globals['_KEYVALUESTORECHUNK']._serialized_end=298
  _globals['_KEYVALUESTOREOUTPUTREQUEST']._serialized_start=300
  _globals['_KEYVALUESTOREOUTPUTREQUEST']._serialized_end=342
  _globals['_KEYVALUESTOREOUTPUTCHUNK']._serialized_start=344
  _globals['_KEYVALUESTOREOUTPUTCHUNK']._serialized_end=386
  _globals['_KEYVALUESTOREREMOVEREQUEST']._serialized_start=388
  _globals['_KEYVALUESTOREREMOVEREQUEST']._serialized_end=430
  _globals['_KEYVALUESTORE']._serialized_start=433
  _globals['_KEYVALUESTORE']._serialized_end=674
# @@protoc_insertion_point(module_scope)



import os
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "generated"))

from .resinsight_classes import *

from .case import Case as Case, EclipseCase as EclipseCase, GeoMechCase as GeoMechCase
from .grid import Grid as Grid
from .instance import Instance as Instance
from .view import View as View
from .project import Project as Project
from .plot import Plot as Plot, PlotWindow as PlotWindow
from .contour_map import (
    EclipseContourMap as EclipseContourMap,
    GeoMechContourMap as GeoMechContourMap,
)
from .well_log_plot import WellLogPlot as WellLogPlot
from .well_path import WellPath as WellPath
from . import well_path_collection
from .simulation_well import SimulationWell as SimulationWell
from .exception import RipsError as RipsError
from .surface import RegularSurface as RegularSurface

from typing import List

__all__: List[str] = []
for key in class_dict():
    __all__.append(key)

# Add classes not in resinsight_classes
__all__.append("Grid")
__all__.append("Instance")

__all__.sort()



###################################################################################
# This example will connect to ResInsight, find bounding box and add a polygon
#
###################################################################################

# Import the ResInsight Processing Server Module
import rips

# Connect to ResInsight
resinsight = rips.Instance.find()
if resinsight is not None:
    # Get a list of all cases
    cases = resinsight.project.cases()

    for c in cases:
        print("Case name: " + c.name)

        # create a polygon which is same a the bounding box in x and y.
        # depth is set to middle of the bounding box
        bbox = c.reservoir_boundingbox()
        depth = bbox.max_z - ((bbox.max_z - bbox.min_z) / 2.0)

        coordinates = []
        coordinates.append([bbox.min_x, bbox.min_y, depth])
        coordinates.append([bbox.max_x, bbox.min_y, depth])
        coordinates.append([bbox.max_x, bbox.max_y, depth])
        coordinates.append([bbox.min_x, bbox.max_y, depth])

        polygon_collection = resinsight.project.descendants(rips.PolygonCollection)[0]
        p = polygon_collection.create_polygon(
            name="{} bounding box".format(c.name), coordinates=coordinates
        )
        print("Coordinates for {}:".format(p.name))
        for coord in p.coordinates:
            print(coord)



###################################################################################
# This example will connect to ResInsight, extract trajectory properties for
# each wells in the project, and extract some properties for each points on the
# trajectory.
#
###################################################################################

import rips


def print_dictionary(title, data):
    # Dictionary of lists of floats.
    keys = list(data.keys())

    print(title)
    for i, properties in enumerate(zip(*data.values()), 1):
        prop_pairs = [
            f"{key.replace('coordinate_', '')}={prop:.3f}"
            for key, prop in zip(keys, properties)
        ]
        print(f"Point {i}: {', '.join(prop_pairs)}")


# Connect to ResInsight
resinsight = rips.Instance.find()
if resinsight is not None:
    # Get a list of all wells
    wells = resinsight.project.well_paths()

    # Find the first case
    cases = resinsight.project.cases()
    c = cases[0] if len(cases) else None

    for well in wells:
        result = well.trajectory_properties(resampling_interval=10.0)

        if c:
            # Convert the result data into points
            positions = [
                list(coord)
                for coord in zip(
                    result["coordinate_x"],
                    result["coordinate_y"],
                    result["coordinate_z"],
                )
            ]

            # Extract some properties
            properties = [
                ("DYNAMIC_NATIVE", "PRESSURE", 0),
                ("STATIC_NATIVE", "FAULTDIST", 0),
            ]

            for property_type, property_name, time_step in properties:
                porosity_model = "MATRIX_MODEL"
                result[property_name] = c.grid_property_for_positions(
                    positions, property_type, property_name, time_step, porosity_model
                )

            title = "Well name: " + well.name
            print_dictionary(title, result)



import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))

import dataroot


def test_visible_cells(rips_instance, initialize_test):
    """Test the visible_cells() method on a view"""
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)

    # Get the first view or create one
    views = case.views()
    if views:
        view = views[0]
    else:
        view = case.create_view()

    # Test visible_cells at time step 0
    visibility = view.visible_cells(time_step=0)

    # Verify we got a list
    assert isinstance(visibility, list)

    # Verify we have data
    assert len(visibility) > 0

    # Verify all values are either 0 or 1
    for v in visibility:
        assert v in [0, 1]

    # Count visible and invisible cells
    visible_count = sum(visibility)
    invisible_count = len(visibility) - visible_count

    # Total cells should match grid size
    assert len(visibility) == visible_count + invisible_count

    # The visibility array should have the correct size (can be all visible, all invisible, or mixed)
    # This depends on the view settings
    assert visible_count + invisible_count > 0


def test_visible_cells_different_timesteps(rips_instance, initialize_test):
    """Test visible_cells() at different time steps"""
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)

    view = case.views()[0] if case.views() else case.create_view()

    # Test at time step 0
    visibility_ts0 = view.visible_cells(time_step=0)
    assert len(visibility_ts0) > 0

    # Test at time step 1 (if it exists)
    visibility_ts1 = view.visible_cells(time_step=1)
    assert len(visibility_ts1) > 0

    # Both should have the same number of cells
    assert len(visibility_ts0) == len(visibility_ts1)



"""
Example: Import Well Log Data to Well Paths

This example demonstrates how to import well log data with measured depth
and multiple channels from Python arrays into ResInsight well paths using
the new well log import API.
"""

import rips


def create_example_well_log_data():
    """Create example well log data with measured depth and multiple channels"""
    # Measured depth values in meters
    measured_depth = [1000, 1200, 1400, 1600, 1800, 2000, 2200, 2400, 2600, 2800]

    # Multiple channel data corresponding to each measured depth
    channel_data = {
        "gamma_ray": [75.2, 82.1, 78.5, 80.3, 85.7, 88.2, 90.1, 87.3, 84.6, 82.8],
        "porosity": [0.12, 0.15, 0.18, 0.22, 0.25, 0.28, 0.24, 0.20, 0.16, 0.14],
        "permeability": [15.5, 22.3, 35.8, 48.2, 65.1, 78.4, 52.9, 38.7, 25.6, 18.9],
        "resistivity": [2.1, 1.8, 3.2, 2.5, 1.9, 1.4, 2.8, 3.1, 2.7, 2.3],
    }

    return measured_depth, channel_data


def main():
    # Connect to ResInsight
    resinsight = rips.Instance.find()
    project = resinsight.project

    # Create a well path from coordinates
    coordinates = [
        [458000, 5934000, -1000],  # X, Y, Z coordinates
        [458100, 5934100, -1500],
        [458200, 5934200, -2000],
        [458300, 5934300, -2500],
        [458400, 5934400, -3000],
    ]

    well_path = project.well_path_collection().import_well_path_from_points(
        name="ExampleWell", coordinates=coordinates
    )

    print(f"Created well path: {well_path.name}")

    # Get example well log data
    measured_depth, channel_data = create_example_well_log_data()

    # Import well log with multiple channels using the new API
    well_log = well_path.add_well_log(
        name="ComprehensiveWellLog",
        measured_depth=measured_depth,
        channel_data=channel_data,
    )

    print(f"Added well log: {well_log.name}")
    print(
        f"  - Measured depth range: {min(measured_depth):.1f} - {max(measured_depth):.1f} m"
    )
    print(f"  - Number of data points: {len(measured_depth)}")
    print(f"  - Channels imported: {', '.join(channel_data.keys())}")

    # You can also import additional well logs with different data
    # For example, a well log with different measured depth intervals
    additional_measured_depth = [1100, 1300, 1500, 1700, 1900, 2100, 2300, 2500, 2700]
    additional_channel_data = {
        "caliper": [8.5, 8.2, 8.7, 8.3, 8.1, 8.4, 8.6, 8.2, 8.3],
        "neutron": [0.18, 0.22, 0.25, 0.28, 0.31, 0.29, 0.26, 0.23, 0.20],
    }

    additional_well_log = well_path.add_well_log(
        name="AdditionalWellLog",
        measured_depth=additional_measured_depth,
        channel_data=additional_channel_data,
    )

    print(f"Added additional well log: {additional_well_log.name}")
    print(
        f"  - MD range: {min(additional_measured_depth):.1f} - {max(additional_measured_depth):.1f} m"
    )
    print(f"  - Channels imported: {', '.join(additional_channel_data.keys())}")


if __name__ == "__main__":
    main()



import sys
import os
import math

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_create_polygon(rips_instance, initialize_test):
    project = rips_instance.project.open(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/10KWithWellLog.rsp"
    )

    c = project.cases()[0]
    bbox = c.reservoir_boundingbox()

    polygon_collection = rips_instance.project.descendants(rips.PolygonCollection)[0]

    coordinates = []
    coordinates.append([bbox.min_x, bbox.min_y, -1000.0])
    coordinates.append([bbox.max_x, bbox.min_y, -1000.0])
    coordinates.append([bbox.max_x, bbox.max_y, -1500.0])
    coordinates.append([bbox.min_x, bbox.max_y, -1500.0])

    name = "{} bounding box".format(c.name)
    p = polygon_collection.create_polygon(name=name, coordinates=coordinates)
    assert p.name == name
    assert len(coordinates) == len(p.coordinates)
    for expected, actual in zip(coordinates, p.coordinates):
        assert len(expected) == len(actual)
        for e, a in zip(expected, actual):
            assert math.isclose(e, a, rel_tol=1e-9, abs_tol=0.0)



import sys
import os
import pytest

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
from rips import RipsError


def test_exception_on_object_method(rips_instance, initialize_test):
    name = "the name"
    nx = 12
    ny = 13
    nz = -1

    with pytest.raises(RipsError, match=r"Invalid grid size"):
        rips_instance.project.create_corner_point_grid(name, nx, ny, nz, [], [], [])



#!/usr/bin/env python3

"""
Example demonstrating how to create valve templates programmatically using the Python API.

This example shows:
1. Creating different types of valve templates (ICD, ICV, AICD)
2. Setting custom parameters for orifice diameter and flow coefficient
3. Using the created templates in well completions
"""

import rips


def main():
    # Connect to ResInsight instance
    resinsight = rips.Instance.find()

    print("Creating valve templates...")

    # Get the valve template collection
    valve_templates = resinsight.project.valve_templates()

    # Check initial number of templates
    initial_templates = valve_templates.valve_definitions()
    print(f"Initial number of valve templates: {len(initial_templates)}")

    # Create an ICD template with default values
    print("\n1. Creating ICD template with default values")
    icd_template = valve_templates.add_template(completion_type="ICD")
    print(f"   Created: {icd_template.name}")
    print(f"   Orifice Diameter: {icd_template.orifice_diameter}")
    print(f"   Flow Coefficient: {icd_template.flow_coefficient}")

    # Create an ICV template with custom values
    print("\n2. Creating ICV template with custom values")
    icv_template = valve_templates.add_template(
        completion_type="ICV",
        orifice_diameter=12.5,
        flow_coefficient=0.85,
        user_label="Custom ICV for High Flow",
    )
    print(f"   Created: {icv_template.name}")
    print(f"   Orifice Diameter: {icv_template.orifice_diameter}")
    print(f"   Flow Coefficient: {icv_template.flow_coefficient}")

    # Create an AICD template (as suggested in the issue)
    print("\n3. Creating AICD template as suggested in issue #12773")
    aicd_template = valve_templates.add_template(
        completion_type="AICD",
        orifice_diameter=7.3,
        flow_coefficient=0.2,
        user_label="Issue Example AICD",
    )
    print(f"   Created: {aicd_template.name}")
    print(f"   Orifice Diameter: {aicd_template.orifice_diameter}")
    print(f"   Flow Coefficient: {aicd_template.flow_coefficient}")

    # Show all valve templates
    current_templates = valve_templates.valve_definitions()
    print(f"\nTotal valve templates now: {len(current_templates)}")
    print("All valve templates:")
    for i, template in enumerate(current_templates):
        print(f"   {i + 1}. {template.name}")

    # Example of using the new template in a completion (requires a loaded case with well paths)
    print("\n4. Example usage in well completion (requires loaded case)")
    try:
        # This assumes you have a case loaded with well paths
        well_paths = resinsight.project.well_paths()
        if well_paths:
            well_path = well_paths[0]
            print(f"   Using well path: {well_path.name}")

            # Add a perforation interval
            perf_interval = well_path.append_perforation_interval(
                start_md=2450, end_md=2500, diameter=0.25, skin_factor=0.1
            )

            # Add a valve using our new template
            valve = perf_interval.add_valve(
                template=aicd_template, start_md=2451, end_md=2499, valve_count=3
            )
            print(f"   Created valve: {valve.name}")
            print(f"   Number of valves in interval: {len(perf_interval.valves())}")
        else:
            print("   No well paths available - skipping completion example")
            print("   Load a case with well paths to see this in action")

    except Exception as e:
        print(f"   Could not create completion example: {e}")
        print("   This is expected if no case/well paths are loaded")

    print("\nExample completed successfully!")
    print("\nAPI Usage Summary:")
    print("- valve_templates.add_template(completion_type='ICD')  # Use defaults")
    print(
        "- valve_templates.add_template(completion_type='AICD', orifice_diameter=7.3, flow_coefficient=0.2)"
    )
    print(
        "- valve_templates.add_template(completion_type='ICV', user_label='My Custom Template')"
    )


if __name__ == "__main__":
    main()



import rips

resinsight = rips.Instance.find()
if resinsight is None:
    raise RuntimeError("No running ResInsight instance found on the expected ports.")

# Create a modeled well path and add well path targets
# The coordinates are based on the Norne case

well_path_coll = resinsight.project.well_path_collection()
well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
well_path.name = "Test Well-1"
well_path.update()

geometry = well_path.well_path_geometry()

reference_point = geometry.reference_point
reference_point[0] = 457196
reference_point[1] = 7322270
reference_point[2] = 2742
geometry.update()  # Commit updates back to ResInsight

# Create the first well target at the reference point
coord = [0, 0, 0]
geometry.append_well_target(coord)

# Append new well targets relative the the reference point
coord = [454.28, 250, -10]
target = geometry.append_well_target(coord)

coord = [1054.28, 250, -50]
target = geometry.append_well_target(coord)

################################
## Create a lateral well path and append it to the main well path
################################

lateral_well = well_path_coll.add_new_object(rips.ModeledWellPath)
lateral_well.name = "Lateral-1"
lateral_well.update()

lateral_geometry = lateral_well.well_path_geometry()

# Only create geometry starting from the first target of the lateral
lateral_geometry.use_auto_generated_target_at_sea_level = False

reference_point = lateral_geometry.reference_point
reference_point[0] = 457550
reference_point[1] = 7322481
reference_point[2] = 2735
lateral_geometry.update()  # Commit updates back to ResInsight

coord = [0, 0, 0]
lateral_geometry.append_well_target(coord)

coord = [454.28, 250, -10]
target = lateral_geometry.append_well_target(coord)

coord = [1054.28, 250, -50]
target = lateral_geometry.append_well_target(coord)

# Append the lateral to the main well path
well_path.append_lateral_from_geometry(lateral_well)



import sys
import os
import tempfile
import pytest
from pathlib import Path

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_create_and_export_surface(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 1

    surface_collection = rips_instance.project.descendants(rips.SurfaceCollection)[0]

    surface = surface_collection.new_surface(case, 5)
    assert surface

    with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
        path = Path(tmpdirname, "mysurface.ts")
        print("Temporary folder: ", path.as_posix())

        surface.export_to_file(path.as_posix())

        assert path.exists()


def test_create_regular_surface(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
    c = rips_instance.project.load_case(path=case_path)
    assert len(c.grids()) == 1

    surface_collection = rips_instance.project.descendants(rips.SurfaceCollection)[0]

    bbox = c.reservoir_boundingbox()
    depth = bbox.max_z - ((bbox.max_z - bbox.min_z) / 2.0)

    origin_x = bbox.min_x
    origin_y = bbox.min_y
    name = "test surface name"
    nx = 20
    ny = 10
    increment_x = (bbox.max_x - bbox.min_x) / float(nx)
    increment_y = (bbox.max_y - bbox.min_y) / float(ny)
    rotation = 23.4

    s = surface_collection.new_regular_surface(
        name=name,
        origin_x=origin_x,
        origin_y=origin_y,
        depth=-depth,
        nx=nx,
        ny=ny,
        increment_x=increment_x,
        increment_y=increment_y,
        rotation=rotation,
    )

    assert s.origin_x == origin_x
    assert s.origin_y == origin_y
    assert s.increment_x == increment_x
    assert s.increment_y == increment_y
    assert s.rotation == rotation
    assert s.nx == nx
    assert s.ny == ny

    with pytest.raises(rips.RipsError, match="Failed to set depth property."):
        s.set_property_as_depth(name="non_existing_property")

    with pytest.raises(
        rips.RipsError, match="Failed to set property: incorrect dimensions."
    ):
        s.set_property("property_too_big", [i for i in range(nx * ny * 2)])

    # Successfully set the depth property
    s.set_property("depth", [i for i in range(nx * ny)])
    s.set_property_as_depth("depth")

    # Change the dimensions of the surface: should invalidate the properties
    s.nx = 30
    s.update()

    # 'depth' property is now gone
    with pytest.raises(rips.RipsError, match="Failed to set depth property."):
        s.set_property_as_depth(name="depth")


def test_create_regular_surface_invalid_values(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
    c = rips_instance.project.load_case(path=case_path)
    assert len(c.grids()) == 1

    surface_collection = rips_instance.project.descendants(rips.SurfaceCollection)[0]

    with pytest.raises(rips.RipsError, match="Invalid nx"):
        surface_collection.new_regular_surface(nx=-1)

    with pytest.raises(rips.RipsError, match="Invalid ny"):
        surface_collection.new_regular_surface(ny=-1)

    with pytest.raises(rips.RipsError, match="Invalid increment X. Must be positive."):
        surface_collection.new_regular_surface(increment_x=-1.0)

    with pytest.raises(rips.RipsError, match="Invalid increment Y. Must be positive."):
        surface_collection.new_regular_surface(increment_y=-1.0)

    with pytest.raises(rips.RipsError, match="Invalid rotation."):
        surface_collection.new_regular_surface(rotation=-1.0)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

import Case_pb2 as Case__pb2
import Definitions_pb2 as Definitions__pb2
import PdmObject_pb2 as PdmObject__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in Project_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class ProjectStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetCurrentCase = channel.unary_unary(
                '/rips.Project/GetCurrentCase',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseRequest.FromString,
                _registered_method=True)
        self.GetSelectedCases = channel.unary_unary(
                '/rips.Project/GetSelectedCases',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseInfoArray.FromString,
                _registered_method=True)
        self.GetAllCaseGroups = channel.unary_unary(
                '/rips.Project/GetAllCaseGroups',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseGroups.FromString,
                _registered_method=True)
        self.GetAllCases = channel.unary_unary(
                '/rips.Project/GetAllCases',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseInfoArray.FromString,
                _registered_method=True)
        self.GetCasesInGroup = channel.unary_unary(
                '/rips.Project/GetCasesInGroup',
                request_serializer=Case__pb2.CaseGroup.SerializeToString,
                response_deserializer=Case__pb2.CaseInfoArray.FromString,
                _registered_method=True)
        self.GetPdmObject = channel.unary_unary(
                '/rips.Project/GetPdmObject',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=PdmObject__pb2.PdmObject.FromString,
                _registered_method=True)


class ProjectServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetCurrentCase(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSelectedCases(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllCaseGroups(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllCases(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCasesInGroup(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPdmObject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ProjectServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetCurrentCase': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCurrentCase,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseRequest.SerializeToString,
            ),
            'GetSelectedCases': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSelectedCases,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseInfoArray.SerializeToString,
            ),
            'GetAllCaseGroups': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllCaseGroups,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseGroups.SerializeToString,
            ),
            'GetAllCases': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllCases,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseInfoArray.SerializeToString,
            ),
            'GetCasesInGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCasesInGroup,
                    request_deserializer=Case__pb2.CaseGroup.FromString,
                    response_serializer=Case__pb2.CaseInfoArray.SerializeToString,
            ),
            'GetPdmObject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPdmObject,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=PdmObject__pb2.PdmObject.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rips.Project', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('rips.Project', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Project(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetCurrentCase(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/rips.Project/GetCurrentCase',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseRequest.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetSelectedCases(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/rips.Project/GetSelectedCases',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseInfoArray.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAllCaseGroups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/rips.Project/GetAllCaseGroups',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseGroups.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAllCases(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/rips.Project/GetAllCases',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseInfoArray.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetCasesInGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/rips.Project/GetCasesInGroup',
            Case__pb2.CaseGroup.SerializeToString,
            Case__pb2.CaseInfoArray.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPdmObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/rips.Project/GetPdmObject',
            Definitions__pb2.Empty.SerializeToString,
            PdmObject__pb2.PdmObject.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import sys
import os
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))

import dataroot


def test_msw_completion_data_perf_lateral(rips_instance, initialize_test):
    """Test MSW completion data retrieval from perf_lateral.rsp project.

    This test loads a project with Multi-Segment Well (MSW) configuration
    and verifies that completion data can be retrieved including MSW-specific
    tables (WELSEGS, COMPSEGS).
    """
    project_path = dataroot.PATH + "/msw-export/project-files/perf_lateral.rsp"
    project = rips_instance.project.open(path=project_path)

    # Get the case
    cases = project.cases()
    assert len(cases) > 0, "Project should contain at least one case"
    case = cases[0]

    # Get well paths
    well_paths = project.well_paths()
    assert len(well_paths) >= 2, "Project should contain at least 2 well paths"

    # Find the well paths by name
    well_y1 = None
    well_y2 = None
    for well in well_paths:
        if well.name == "Well-1 Y1":
            well_y1 = well
        elif well.name == "Well-1 Y2":
            well_y2 = well

    assert well_y1 is not None, "Well-1 Y1 should exist in project"
    assert well_y2 is not None, "Well-1 Y2 should exist in project"

    # Test completion data for Well-1 Y1
    completion_data_y1 = well_y1.completion_data(case.id)

    # Verify WELSPECS data
    welspecs = completion_data_y1.welspecs
    assert len(welspecs) > 0, "Should have WELSPECS data for Well-1 Y1"

    welspec = welspecs[0]
    assert welspec.well_name == "Well-1", "Well name should be Well-1"
    assert welspec.group_name == "PROD_N", "Group name should be PROD_N"
    assert welspec.phase == "OIL", "Phase should be OIL"

    # Verify COMPDAT data (perforation intervals)
    compdat = completion_data_y1.compdat
    assert len(compdat) > 0, "Should have COMPDAT data for Well-1 Y1"

    # Check that we have MD information in COMPDAT
    has_md_data = False
    for comp in compdat:
        if comp.HasField("start_md") and comp.HasField("end_md"):
            has_md_data = True
            assert comp.start_md > 0, "Start MD should be positive"
            assert comp.end_md > comp.start_md, "End MD should be greater than start MD"
            break

    assert has_md_data, "Should have MD (measured depth) data in COMPDAT"

    # Verify MSW-specific data: WELSEGS
    welsegs = completion_data_y1.welsegs
    assert len(welsegs) > 0, "Should have WELSEGS data for MSW well Well-1 Y1"

    # Check WELSEGS header (first entry)
    welsegs_entry = welsegs[0]
    assert (
        welsegs_entry.header.well_name == "Well-1"
    ), "WELSEGS well name should be Well-1"
    assert (
        welsegs_entry.header.top_depth > 0
    ), "Should have positive top depth in WELSEGS"
    assert welsegs_entry.header.top_length >= 0, "Should have top length in WELSEGS"
    assert welsegs_entry.header.info_type in [
        "ABS",
        "INC",
    ], "Should have valid info type"

    # Check WELSEGS rows (segment definitions)
    assert len(welsegs_entry.row) > 0, "Should have segment rows in WELSEGS"
    for row in welsegs_entry.row:
        assert row.segment_1 > 0, "Segment 1 should be positive"
        assert row.segment_2 > 0, "Segment 2 should be positive"

    # Verify MSW-specific data: COMPSEGS
    compsegs = completion_data_y1.compsegs
    assert len(compsegs) > 0, "Should have COMPSEGS data for MSW well Well-1 Y1"

    # Check COMPSEGS entries
    for compseg in compsegs:
        assert compseg.i > 0, "Grid I should be positive"
        assert compseg.j > 0, "Grid J should be positive"
        assert compseg.k > 0, "Grid K should be positive"
        assert compseg.branch >= 0, "Branch should be non-negative"


def test_msw_completion_data_unified(rips_instance, initialize_test):
    """Test unified MSW completion data retrieval for multiple wells.

    This test verifies that completion data can be retrieved for multiple
    MSW wells in a single call using the completion_data_unified method.
    """
    project_path = dataroot.PATH + "/msw-export/project-files/perf_lateral.rsp"
    project = rips_instance.project.open(path=project_path)

    # Get the case
    cases = project.cases()
    assert len(cases) > 0, "Project should contain at least one case"
    case = cases[0]

    # Get well path collection
    well_path_collection = project.well_path_collection()

    # Get unified completion data for all wells
    unified_data = well_path_collection.completion_data_unified(case_id=case.id)

    # Verify WELSPECS
    assert len(unified_data.welspecs) > 0, "Should have WELSPECS in unified data"

    # Verify COMPDAT
    assert len(unified_data.compdat) > 0, "Should have COMPDAT in unified data"

    # Verify WELSEGS (MSW-specific)
    assert len(unified_data.welsegs) > 0, "Should have WELSEGS in unified MSW data"

    # Count wells in WELSEGS
    well_names_in_welsegs = set()
    for welsegs_entry in unified_data.welsegs:
        well_names_in_welsegs.add(welsegs_entry.header.well_name)

    assert (
        len(well_names_in_welsegs) >= 1
    ), "Should have at least one well in WELSEGS data"

    # Verify COMPSEGS (MSW-specific)
    assert len(unified_data.compsegs) > 0, "Should have COMPSEGS in unified MSW data"

    # Check that COMPSEGS has valid grid coordinates
    for compseg in unified_data.compsegs:
        assert compseg.i > 0, "Grid I should be positive"
        assert compseg.j > 0, "Grid J should be positive"
        assert compseg.k > 0, "Grid K should be positive"


def test_msw_export(rips_instance, initialize_test):
    """Test MSW export functionality.

    This test verifies that MSW data can be exported to file using the
    export_msw method on a case.
    """
    project_path = dataroot.PATH + "/msw-export/project-files/perf_lateral.rsp"
    project = rips_instance.project.open(path=project_path)

    # Get the case
    cases = project.cases()
    assert len(cases) > 0, "Project should contain at least one case"
    case = cases[0]

    # Get well paths
    well_paths = project.well_paths()
    well_y1 = None
    for well in well_paths:
        if well.name == "Well-1 Y1":
            well_y1 = well
            break

    assert well_y1 is not None, "Well-1 Y1 should exist in project"

    # Set export folder
    export_folder = tempfile.gettempdir()
    rips_instance.set_export_folder(export_type="COMPLETIONS", path=export_folder)

    # Export MSW data
    case.export_msw(well_path=well_y1.name)

    # Note: The actual file creation validation would require checking
    # the export folder, which may be platform-dependent and require
    # additional file system checks


def test_msw_settings_read(rips_instance, initialize_test):
    """Test reading MSW settings from project.

    This test verifies that MSW settings configured in the project
    can be read through the Python API.
    """
    project_path = dataroot.PATH + "/msw-export/project-files/perf_lateral.rsp"
    project = rips_instance.project.open(path=project_path)

    # Get well paths
    well_paths = project.well_paths()
    well_y1 = None
    for well in well_paths:
        if well.name == "Well-1 Y1":
            well_y1 = well
            break

    assert well_y1 is not None, "Well-1 Y1 should exist in project"

    # Get MSW settings
    msw_settings = well_y1.msw_settings()

    # Verify MSW settings values from the project file
    # According to the project analysis, the liner diameter is 0.152m
    assert msw_settings.liner_diameter == 0.152, "Liner diameter should be 0.152m"
    assert msw_settings.roughness_factor == 1e-05, "Roughness factor should be 1e-05"

    # Check pressure drop setting
    assert msw_settings.pressure_drop == "HF-", "Pressure drop should be HF-"

    # Check max segment length setting
    assert msw_settings.max_segment_length == 200.0, "Max segment length should be 200m"

    # Verify completion settings
    completion_settings = well_y1.completion_settings()
    assert (
        completion_settings.well_name_for_export == "Well-1"
    ), "Well name for export should be Well-1"
    assert (
        completion_settings.group_name_for_export == "PROD_N"
    ), "Group name should be PROD_N"
    assert completion_settings.well_type_for_export == "OIL", "Well type should be OIL"
    assert (
        completion_settings.allow_well_cross_flow
    ), "Allow well cross flow should be True"



######################################################################
# This script creates a regular surface for each case
######################################################################
import rips
import math


def create_x_surface(nx, ny):
    surface = []
    for j in range(ny):
        for i in range(nx):
            surface.append(float(i))
    return surface


def create_wave_surface(nx, ny):
    # Fill the coordinate and wave pattern arrays
    surface = []
    for j in range(ny):
        for i in range(nx):
            # Create wave pattern - combining multiple sine waves
            x = -5 + 10 * i / (nx - 1)
            y = -5 + 10 * j / (ny - 1)
            offset = (
                math.sin(x**2 + y**2)
                + 0.5 * math.sin(2 * x) * math.cos(2 * y)
                + 25.0 * math.cos(5 * x + 2 * y)
            )
            surface.append(-depth + offset)
    return surface


resinsight = rips.Instance.find()

project = resinsight.project


if resinsight is not None:
    # Get a list of all cases
    cases = resinsight.project.cases()

    for c in cases:
        bbox = c.reservoir_boundingbox()
        depth = bbox.max_z - ((bbox.max_z - bbox.min_z) / 2.0)

        origin_x = bbox.min_x
        origin_y = bbox.min_y

        name = "{} surface".format(c.name)

        nx = 200
        ny = 100

        increment_x = (bbox.max_x - bbox.min_x) / float(nx)
        increment_y = (bbox.max_y - bbox.min_y) / float(ny)

        surface_collection = resinsight.project.descendants(rips.SurfaceCollection)[0]

        # Create a surface at a given depth
        s = surface_collection.new_regular_surface(
            name=name,
            origin_x=origin_x,
            origin_y=origin_y,
            depth=-depth,
            nx=nx,
            ny=ny,
            increment_x=increment_x,
            increment_y=increment_y,
        )

        # Rotate the resulting surface
        s.rotation = 45.0
        s.update()

        # Add one property
        s.set_property("first_property", create_x_surface(nx, ny))

        # Add a wave surface
        s.set_property("wave", create_wave_surface(nx, ny))

        # Use the wave as depth for the surface
        s.set_property_as_depth("wave")



import uuid
import grpc
from typing import List, Optional, Union, cast

import SimulatorTables_pb2
import SimulatorTables_pb2_grpc

import PdmObject_pb2

from .pdmobject import add_method
from .resinsight_classes import WellPathCollection, WellPath
from .project import Project


@add_method(WellPathCollection)
def __custom_init__(
    self: WellPathCollection, pb2_object: PdmObject_pb2.PdmObject, channel: grpc.Channel
) -> None:
    self.__well_path_stub = SimulatorTables_pb2_grpc.WellPathStub(channel)


@add_method(WellPathCollection)
def import_well_path_from_points(
    self: WellPathCollection, name: str, coordinates: List[List[float]]
) -> object:
    """Create a well path from a list of XYZ coordinates.

    Arguments:
        name (str): Name of the well path
        coordinates (List[List[float]]): List of [x, y, z] coordinate triplets

    Returns:
        RimPointBasedWellPath: The created well path object

    Raises:
        ValueError: If coordinates are invalid or have wrong dimensions
    """

    if not name:
        raise ValueError("Name cannot be empty")

    if not coordinates:
        raise ValueError("Coordinates list cannot be empty")

    # Validate coordinate format
    for i, coord in enumerate(coordinates):
        if not isinstance(coord, (list, tuple)) or len(coord) != 3:
            raise ValueError(
                f"Coordinate at index {i} must be a list/tuple of 3 values [x, y, z]"
            )

        # Convert to float to validate numeric values
        try:
            float(coord[0])
            float(coord[1])
            float(coord[2])
        except (ValueError, TypeError):
            raise ValueError(
                f"All coordinate values must be numeric. Invalid coordinate at index {i}: {coord}"
            )

    # Extract separate x, y, z arrays
    x_coords = [float(coord[0]) for coord in coordinates]
    y_coords = [float(coord[1]) for coord in coordinates]
    z_coords = [float(coord[2]) for coord in coordinates]

    # Generate temporary keys with shared UUID for key-value store
    shared_uuid = uuid.uuid4()
    x_key = f"{shared_uuid}_coordinate_x"
    y_key = f"{shared_uuid}_coordinate_y"
    z_key = f"{shared_uuid}_coordinate_z"

    # Store coordinates in key-value store
    project = self.ancestor(Project)
    if not project:
        raise RuntimeError("Could not find parent project")

    project.set_key_values(x_key, x_coords)
    project.set_key_values(y_key, y_coords)
    project.set_key_values(z_key, z_coords)

    try:
        # Call internal method to create the well path
        well_path = self.import_well_path_from_points_internal(
            name=name,
            coordinate_x_key=x_key,
            coordinate_y_key=y_key,
            coordinate_z_key=z_key,
        )

        return well_path

    finally:
        # Clean up temporary keys from key-value store
        project.remove_key_values(x_key)
        project.remove_key_values(y_key)
        project.remove_key_values(z_key)


@add_method(WellPathCollection)
def completion_data_unified(
    self: WellPathCollection,
    wells: Optional[Union[List[WellPath], List[str]]] = None,
    case_id: Optional[int] = None,
) -> SimulatorTables_pb2.SimulatorTableData:
    """Get unified completion data for multiple wells.

    This method merges completion data (COMPDAT, WELSPECS, WELSEGS, COMPSEGS,
    WSEGVALV, WSEGAICD) from multiple wells into a single response.

    Arguments:
        wells: List of WellPath objects or well names. If None, uses all wells in collection.
        case_id: ID of the case to use. If None, tries to determine from context.

    Returns:
        SimulatorTableData containing merged completion data from all specified wells

    Raises:
        ValueError: If case_id cannot be determined or wells list is invalid
        RuntimeError: If unable to access required data
    """
    # Determine case_id if not provided
    if case_id is None:
        project = self.ancestor(Project)
        if not project:
            raise RuntimeError("Could not find parent project")

        # Try to get case_id from the first available case
        cases = project.cases()
        if not cases:
            raise RuntimeError("No cases available in project")
        case_id = cases[0].case_id

    # Handle wells parameter
    if wells is None:
        # Get all wells in collection
        well_names = [wp.name for wp in self.well_paths()]
    elif all(isinstance(w, str) for w in wells):
        # List of well names
        well_names = cast(List[str], wells)
    elif all(hasattr(w, "name") for w in wells):
        # List of WellPath objects
        well_names = [w.name for w in wells if hasattr(w, "name")]
    else:
        raise ValueError(
            "wells parameter must be a list of WellPath objects or strings"
        )

    if not well_names:
        raise ValueError("No wells specified or found in collection")

    sim_tab_req = SimulatorTables_pb2.SimulatorTableUnifiedRequest(
        wellpath_names=well_names, case_id=case_id
    )
    return self.__well_path_stub.GetCompletionDataUnified(sim_tab_req)



import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot

import pytest


def test_10k(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
        case_root_path + "/wellpath_b.dev",
    ]
    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    assert len(well_path_names) == 2
    wells = rips_instance.project.well_paths()
    assert len(wells) == 2
    assert wells[0].name == "Well Path A"
    assert wells[1].name == "Well Path B"


def test_10k_import_well(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
        case_root_path + "/wellpath_b.dev",
    ]

    well_a = rips_instance.project.well_path_collection().import_well_path(
        well_path_files[0]
    )
    assert well_a.name == "Well Path A"

    well_b = rips_instance.project.well_path_collection().import_well_path(
        well_path_files[1]
    )
    assert well_b.name == "Well Path B"

    wells = rips_instance.project.well_paths()
    assert len(wells) == 2
    assert wells[0].name == "Well Path A"
    assert wells[1].name == "Well Path B"


def test_10k_intersection(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
    ]

    view = case.create_view()
    view.set_time_step(1)

    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    assert len(well_path_names) == 1
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    # Add a curve intersection based on the well path
    intersection_coll = rips_instance.project.descendants(rips.IntersectionCollection)[
        0
    ]
    well_path_intersection = intersection_coll.add_new_object(rips.CurveIntersection)
    well_path_intersection.type = "CS_WELL_PATH"
    well_path_intersection.well_path = well_path
    well_path_intersection.update()

    # Three coords per triangle
    geometry = well_path_intersection.geometry()
    coord_count = len(geometry.x_coords)
    assert coord_count == 13254

    # One value per triangle
    geometry_result_values = well_path_intersection.geometry_result()
    result_count = len(geometry_result_values.values)
    assert result_count == 4418

    # Three coords per triangle, one result value per triangle
    assert (result_count * 3) == coord_count


def test_empty_well_intersection(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)

    view = case.create_view()
    view.set_time_step(1)

    # Add a curve intersection based on the well path
    intersection_coll = rips_instance.project.descendants(rips.IntersectionCollection)[
        0
    ]

    well_path_intersection = intersection_coll.add_new_object(rips.CurveIntersection)
    well_path_intersection.type = "CS_WELL_PATH"
    well_path_intersection.well_path = None
    well_path_intersection.update()

    # Test with empty geometry.
    with pytest.raises(rips.RipsError):
        well_path_intersection.geometry()

    with pytest.raises(rips.RipsError):
        well_path_intersection.geometry_result()


def test_10k_intersection_trajectory_properties(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
    ]

    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    assert len(well_path_names) == 1
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    result = well_path.trajectory_properties(resampling_interval=10.0)
    assert result

    assert "coordinate_x" in result
    assert "coordinate_y" in result
    assert "coordinate_z" in result
    assert "measured_depth" in result
    assert "azimuth" in result
    assert "inclination" in result
    assert "dogleg" in result


def test_10k_intersection_add_well_perforation_interval_with_valves(
    rips_instance, initialize_test
):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
    ]

    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    assert len(well_path_names) == 1
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    result = well_path.trajectory_properties(resampling_interval=10.0)

    measured_depths = result["measured_depth"]

    start_md = measured_depths[len(measured_depths) - 3]
    end_md = measured_depths[len(measured_depths) - 2]
    diameter = 0.25
    skin_factor = 0.1

    perf_interval = well_path.append_perforation_interval(
        start_md, end_md, diameter, skin_factor
    )

    assert perf_interval.start_measured_depth == start_md
    assert perf_interval.end_measured_depth == end_md
    assert perf_interval.diameter == diameter
    assert perf_interval.skin_factor == skin_factor

    valve_templates = rips_instance.project.valve_templates()
    assert len(valve_templates.valve_definitions()) == 3

    assert len(perf_interval.valves()) == 0

    valve_start_md = start_md + 1
    valve_end_md = end_md - 1
    valve_count = 1
    valve = perf_interval.add_valve(
        template=valve_templates.valve_definitions()[0],
        start_md=valve_start_md,
        end_md=valve_end_md,
        valve_count=valve_count,
    )

    assert valve
    assert valve.name == "1 AICD: 2451 - 2459"

    assert len(perf_interval.valves()) == 1


def test_10k_intersection_add_well_perforation_interval_with_invalid_valves(
    rips_instance, initialize_test
):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
    ]

    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    assert len(well_path_names) == 1
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    result = well_path.trajectory_properties(resampling_interval=10.0)

    measured_depths = result["measured_depth"]

    start_md = measured_depths[len(measured_depths) - 3]
    end_md = measured_depths[len(measured_depths) - 2]
    diameter = 0.25
    skin_factor = 0.1

    perf_interval = well_path.append_perforation_interval(
        start_md, end_md, diameter, skin_factor
    )

    valve_templates = rips_instance.project.valve_templates()

    with pytest.raises(rips.RipsError):
        # end_md < start_md
        perf_interval.add_valve(
            template=valve_templates.valve_definitions()[0],
            start_md=1000,
            end_md=800,
            valve_count=1,
        )

    with pytest.raises(rips.RipsError):
        # zero valves
        perf_interval.add_valve(
            template=valve_templates.valve_definitions()[0],
            start_md=400,
            end_md=800,
            valve_count=0,
        )


def test_valve_template_creation(rips_instance, initialize_test):
    """Test creating valve templates through Python API"""
    valve_templates = rips_instance.project.valve_templates()

    # Check initial state - should have 3 default templates
    initial_count = len(valve_templates.valve_definitions())
    assert initial_count == 3

    # Test creating ICD template with default values
    icd_template = valve_templates.add_template(completion_type="ICD")
    assert icd_template is not None
    assert icd_template.orifice_diameter == 8.0  # Default value
    assert icd_template.flow_coefficient == 0.7  # Default value
    assert len(valve_templates.valve_definitions()) == initial_count + 1

    # Test creating ICV template with custom values
    icv_template = valve_templates.add_template(
        completion_type="ICV",
        orifice_diameter=10.5,
        flow_coefficient=0.9,
        user_label="Custom ICV Template",
    )
    assert icv_template is not None
    assert icv_template.orifice_diameter == 10.5
    assert icv_template.flow_coefficient == 0.9
    assert "Custom ICV Template" in icv_template.name
    assert len(valve_templates.valve_definitions()) == initial_count + 2

    # Test creating AICD template
    aicd_template = valve_templates.add_template(
        completion_type="AICD",
        orifice_diameter=7.3,
        flow_coefficient=0.2,
        user_label="Test AICD",
    )
    assert aicd_template is not None
    assert aicd_template.orifice_diameter == 7.3
    assert aicd_template.flow_coefficient == 0.2
    assert "Test AICD" in aicd_template.name
    assert len(valve_templates.valve_definitions()) == initial_count + 3

    # Test that the new templates can be used in valves
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    rips_instance.project.load_case(path=case_path)
    well_path_files = [case_root_path + "/wellpath_a.dev"]
    rips_instance.project.import_well_paths(well_path_files)
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    # Add perforation with our new template
    perf_interval = well_path.append_perforation_interval(2450, 2460, 0.25, 0.1)
    valve = perf_interval.add_valve(
        template=icd_template,
        start_md=2451,
        end_md=2459,
        valve_count=1,
    )
    assert valve is not None
    assert len(perf_interval.valves()) == 1


def test_valve_template_invalid_completion_type(rips_instance, initialize_test):
    """Test error handling for invalid completion types"""
    valve_templates = rips_instance.project.valve_templates()

    # Test invalid completion type
    with pytest.raises(rips.RipsError):
        valve_templates.add_template(completion_type="INVALID_TYPE")



###################################################################################
# This example will connect to ResInsight, retrieve a list of
# simulation wells for a case and get the accumulated perforated length for all
# simulation wells per timestep. If a well is closed at a given timestep, the
# accumulated perforated length will be zero.
###################################################################################

# Import the ResInsight Processing Server Module
import rips
import sys

# Connect to ResInsight
resinsight = rips.Instance.find()
if resinsight is None:
    sys.exit("ResInsight is not running. Please start ResInsight and try again.")

# Get a list of all wells
cases = resinsight.project.cases()
if len(cases) == 0:
    sys.exit("No cases found in the project. Please open a case and try again.")

case = cases[0]
print("Using Case: " + case.name)

timesteps = case.time_steps()

# store results in a dictionary,
# use well name as the key and a list of the wells accumulated perforation
# lengths for each timestep as the value
results = {}

# loop over all available simulation wells
sim_wells = case.simulation_wells()
for sim_well in sim_wells:
    acclengths = []
    for tidx, timestep in enumerate(timesteps):
        acclengths.append(sim_well.accumulated_perforation_length(tidx))
    results[sim_well.name] = acclengths

# Print header
header = ["Timestep"]
for tidx, ts in enumerate(timesteps):
    header.append("%d/%d/%d" % (ts.day, ts.month, ts.year))
print(";".join(header))

# Print the results
for well_name, lengths in results.items():
    line = [well_name]
    for idx, length in enumerate(lengths):
        line.append(str(length))
    print(";".join(line))



###################################################################################
# This example will connect to ResInsight, retrieve a list of wells, print info and
#  get the completion data for the first well found
#
###################################################################################

# Import the ResInsight Processing Server Module
import rips


# helper method to format printing for optional fields
def fieldValueOrDefaultText(grpc_object, optional_field_name: str):
    if not grpc_object.HasField(optional_field_name):
        return "1*"
    return str(grpc_object.__getattribute__(optional_field_name))


# Connect to ResInsight
resinsight = rips.Instance.find()
if resinsight is None:
    exit(1)

# Get a list of all wells
wells = resinsight.project.well_paths()

# Get a list of all cases
cases = resinsight.project.cases()

# Use the first one
the_case = cases[0]
print("Using case " + the_case.name)

print("Got " + str(len(wells)) + " well paths: ")
for well in wells:
    print("Well path name: " + well.name + "\n\n")

    completion_data = well.completion_data(the_case.id)

    print("WELSPECS")

    welspecs = completion_data.welspecs
    welspecl_entries = []

    for line in welspecs:
        # Check if grid_name is present and not empty
        has_grid_name = (
            hasattr(line, "grid_name")
            and line.HasField("grid_name")
            and line.grid_name.strip()
        )

        if has_grid_name:
            # Store for WELSPECL section
            welspecl_entries.append(line)
        else:
            # Output as WELSPECS
            txt = line.well_name + "  "
            txt += line.group_name + "  "
            txt += str(line.grid_i) + "  "
            txt += str(line.grid_j) + "  "
            txt += fieldValueOrDefaultText(line, "bhp_depth") + "  "
            txt += line.phase + "  "
            txt += fieldValueOrDefaultText(line, "drainage_radius") + "  "
            txt += fieldValueOrDefaultText(line, "inflow_equation") + "  "
            txt += fieldValueOrDefaultText(line, "auto_shut_in") + "  "
            txt += fieldValueOrDefaultText(line, "cross_flow") + "  "
            txt += fieldValueOrDefaultText(line, "pvt_num") + "  "
            txt += fieldValueOrDefaultText(line, "hydrostatic_density_calc") + "  "
            txt += fieldValueOrDefaultText(line, "fip_region") + "  "
            txt += "/"

            print(txt)

    print("/\n")

    # Output WELSPECL section if there are entries with grid_name
    if len(welspecl_entries) > 0:
        print("WELSPECL")

        for line in welspecl_entries:
            txt = line.well_name + "  "
            txt += line.group_name + "  "
            txt += line.grid_name + "  "
            txt += str(line.grid_i) + "  "
            txt += str(line.grid_j) + "  "
            txt += fieldValueOrDefaultText(line, "bhp_depth") + "  "
            txt += line.phase + "  "
            txt += fieldValueOrDefaultText(line, "drainage_radius") + "  "
            txt += fieldValueOrDefaultText(line, "inflow_equation") + "  "
            txt += fieldValueOrDefaultText(line, "auto_shut_in") + "  "
            txt += fieldValueOrDefaultText(line, "cross_flow") + "  "
            txt += fieldValueOrDefaultText(line, "pvt_num") + "  "
            txt += fieldValueOrDefaultText(line, "hydrostatic_density_calc") + "  "
            txt += fieldValueOrDefaultText(line, "fip_region") + "  "
            txt += "/"

            print(txt)

        print("/\n")

    compdat = completion_data.compdat

    complump_entries = []
    compdatl_entries = []

    print("COMPDAT")

    for line in compdat:
        # Check if grid_name is present and not empty
        has_grid_name = (
            hasattr(line, "grid_name")
            and line.HasField("grid_name")
            and line.grid_name.strip()
        )

        if has_grid_name:
            # Store for COMPDATL section
            compdatl_entries.append(line)
        else:
            # Output as COMPDAT
            txt = ""
            complump = ""

            if line.HasField("start_md"):
                txt += "-- Perforation MD In " + str(line.start_md)
                txt += ", MD Out " + str(line.end_md) + "--\n"

            txt += "   "
            txt += line.well_name + "  "
            txt += str(line.grid_i) + "  "
            txt += str(line.grid_j) + "  "
            txt += str(line.upper_k) + "  "
            txt += str(line.lower_k) + "  "
            txt += line.open_shut_flag + "  "
            txt += fieldValueOrDefaultText(line, "saturation") + "  "
            txt += str(line.transmissibility) + "  "
            txt += str(line.diameter) + "  "
            txt += str(line.kh) + "  "
            txt += fieldValueOrDefaultText(line, "skin_factor") + "  "
            txt += fieldValueOrDefaultText(line, "d_factor") + "  "
            txt += "'%s'" % line.direction
            txt += " /"

            if (line.HasField("completion_number")) and (line.completion_number > 0):
                complump += "   "
                complump += line.well_name + "  "
                complump += str(line.grid_i) + "  "
                complump += str(line.grid_j) + "  "
                complump += str(line.upper_k) + "  "
                complump += str(line.lower_k) + "  "
                complump += str(line.completion_number) + "  "
                complump += " /"

                complump_entries.append(complump)

            print(txt)

    print("/\n")

    # Output COMPDATL section if there are entries with grid_name
    if len(compdatl_entries) > 0:
        print("COMPDATL")

        for line in compdatl_entries:
            txt = ""

            if line.HasField("start_md"):
                txt += "-- Perforation MD In " + str(line.start_md)
                txt += ", MD Out " + str(line.end_md) + "--\n"

            txt += "   "
            txt += line.well_name + "  "
            txt += line.grid_name + "  "
            txt += str(line.grid_i) + "  "
            txt += str(line.grid_j) + "  "
            txt += str(line.upper_k) + "  "
            txt += str(line.lower_k) + "  "
            txt += line.open_shut_flag + "  "
            txt += fieldValueOrDefaultText(line, "saturation") + "  "
            txt += str(line.transmissibility) + "  "
            txt += str(line.diameter) + "  "
            txt += str(line.kh) + "  "
            txt += fieldValueOrDefaultText(line, "skin_factor") + "  "
            txt += fieldValueOrDefaultText(line, "d_factor") + "  "
            txt += "'%s'" % line.direction
            txt += " /"

            print(txt)

        print("/\n")

    if len(complump_entries) > 0:
        print("COMPLUMP")
        for complump_entry in complump_entries:
            print(complump_entry)
        print("/\n")
    else:
        print("-- No COMPLUMP entries --\n")

    print("WELSEGS")
    welsegs = completion_data.welsegs
    if (welsegs is None) or (len(welsegs) == 0):
        print("  -- No WELSEGS data --\n")
    else:
        for welsegs_entry in welsegs:
            # Print WELSEGS header
            header = welsegs_entry.header
            txt = "-- Header: " + header.well_name + "\n"
            txt += "   " + header.well_name + "  "
            txt += str(header.top_depth) + "  "
            txt += str(header.top_length) + "  "
            txt += fieldValueOrDefaultText(header, "wellbore_volume") + "  "
            txt += header.info_type + "  "
            txt += fieldValueOrDefaultText(header, "pressure_components") + "  "
            txt += fieldValueOrDefaultText(header, "flow_model")
            txt += " /"
            print(txt)

            # Print WELSEGS segment rows
            for row in welsegs_entry.row:
                txt = "   "
                txt += str(row.segment_1) + "  "
                txt += str(row.segment_2) + "  "
                txt += str(row.branch) + "  "
                txt += str(row.join_segment) + "  "
                txt += str(row.length) + "  "
                txt += str(row.depth) + "  "
                txt += fieldValueOrDefaultText(row, "diameter") + "  "
                txt += fieldValueOrDefaultText(row, "roughness") + "  "
                txt += " /"
                print(txt)

    print("/")

    # Print COMPSEGS (MSW completion segments)
    print("\nCOMPSEGS")
    compsegs = completion_data.compsegs
    compsegl_entries = []

    if (compsegs is None) or (len(compsegs) == 0):
        print("  -- No COMPSEGS data --")
    else:
        for compseg in compsegs:
            # Check if grid_name is present and not empty
            has_grid_name = (
                hasattr(compseg, "grid_name")
                and compseg.HasField("grid_name")
                and compseg.grid_name.strip()
            )

            if has_grid_name:
                # Store for COMPSEGL section
                compsegl_entries.append(compseg)
            else:
                # Output as COMPSEGS
                txt = "   "
                txt += str(compseg.i) + "  "
                txt += str(compseg.j) + "  "
                txt += str(compseg.k) + "  "
                txt += str(compseg.branch) + "  "
                txt += str(compseg.distance_start) + "  "
                txt += str(compseg.distance_end) + "  "
                txt += " /"
                print(txt)

    print("/\n")

    # Output COMPSEGL section if there are entries with grid_name
    if len(compsegl_entries) > 0:
        print("COMPSEGL")

        for compseg in compsegl_entries:
            txt = "   "
            txt += compseg.grid_name + "  "
            txt += str(compseg.i) + "  "
            txt += str(compseg.j) + "  "
            txt += str(compseg.k) + "  "
            txt += str(compseg.branch) + "  "
            txt += str(compseg.distance_start) + "  "
            txt += str(compseg.distance_end) + "  "
            txt += " /"
            print(txt)

        print("/\n")



import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot

# The generated data is written to a .gitignore'd folder 'completion_export_output'.
# The reference data is stored in 'completion_export_reference'.
# If changes are made to the export functionality that are intended to change the output,
# the reference data must be updated accordingly.


def normalize_content(text):
    """Normalize content by removing header until '-- Grid Model:' line and excluding file path from compare."""
    lines = text.split("\n")

    # Find the line containing "-- Grid Model:" and skip it and the following line
    start_index = 0
    for i, line in enumerate(lines):
        if "-- Grid Model:" in line:
            start_index = i + 2  # Skip "-- Grid Model:" and the line after it
            break

    if start_index >= len(lines):
        return ""

    normalized_lines = lines[start_index:]
    return "\n".join(normalized_lines)


def compare_exported_files_with_reference(
    export_folder, reference_folder, file_type_description
):
    """Compare all exported files with their corresponding reference files.

    Args:
        export_folder: Path to folder containing exported files
        reference_folder: Path to folder containing reference files
        file_type_description: Description for error messages (e.g., "unified export", "split by well")
    """
    exported_files = [
        f
        for f in os.listdir(export_folder)
        if os.path.isfile(os.path.join(export_folder, f))
    ]

    for filename in exported_files:
        export_file_path = os.path.join(export_folder, filename)
        reference_file_path = os.path.join(reference_folder, filename)
        compare_with_reference(
            export_file_path,
            reference_file_path,
            f"{file_type_description} file '{filename}'",
        )


def compare_with_reference(
    export_file_path, reference_file_path, file_description="file"
):
    """Compare an export file with reference data

    Args:
        export_file_path: Path to the exported file
        reference_file_path: Path to the reference file
        file_description: Description of the file for error messages

    Raises:
        AssertionError: If files don't match or don't exist
    """
    assert os.path.exists(
        export_file_path
    ), f"Export {file_description} does not exist: {export_file_path}"

    with open(export_file_path, "r") as f:
        content = f.read()

    if os.path.exists(reference_file_path):
        with open(reference_file_path, "r") as ref_f:
            ref_content = ref_f.read()

        normalized_content = normalize_content(content)
        normalized_ref_content = normalize_content(ref_content)

        assert normalized_content == normalized_ref_content, (
            f"Export {file_description} differs from reference data. "
            f"Export length: {len(normalized_content)}, Reference length: {len(normalized_ref_content)}"
        )


def test_export_completion_files_unified(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    project_path = case_root_path + "/well_completions_export.rsp"
    project = rips_instance.project.open(path=project_path)

    export_folder = os.path.abspath(
        case_root_path + "/completion_export_output/unified_export"
    )
    os.makedirs(export_folder, exist_ok=True)

    try:
        rips_instance.set_export_folder(export_type="COMPLETIONS", path=export_folder)

        case = project.cases()[0]

        well_names_to_use = ["Well-1"]

        case.export_well_path_completions(
            time_step=0,  # Use time step 0 instead of 1
            well_path_names=well_names_to_use,
            file_split="UNIFIED_FILE",
            include_perforations=True,
            include_fishbones=True,
            export_welspec=True,
            export_comments=False,
        )

        # Compare exported files with reference data
        reference_folder = (
            case_root_path + "/completion_export_reference/unified_export"
        )
        compare_exported_files_with_reference(
            export_folder, reference_folder, "unified export"
        )

    finally:
        pass  # Keep exported files in the output directory


def test_export_completion_files_split_by_well(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    project_path = case_root_path + "/well_completions_export.rsp"
    project = rips_instance.project.open(path=project_path)

    export_folder = os.path.abspath(
        case_root_path + "/completion_export_output/split_by_well"
    )
    os.makedirs(export_folder, exist_ok=True)

    try:
        rips_instance.set_export_folder(export_type="COMPLETIONS", path=export_folder)

        case = project.cases()[0]

        # Use Well-1 for all tests
        test_wells = ["Well-1"]

        case.export_well_path_completions(
            time_step=0,  # Use time step 0 instead of 1
            well_path_names=test_wells,
            file_split="SPLIT_ON_WELL",
            include_perforations=True,
            include_fishbones=False,  # This matches the reference generation script
            export_comments=False,
        )

        # Compare exported files with reference data
        reference_folder = case_root_path + "/completion_export_reference/split_by_well"
        compare_exported_files_with_reference(
            export_folder, reference_folder, "split by well"
        )

    finally:
        pass  # Keep exported files in the output directory



import rips


def test_diameter_roughness_intervals(rips_instance, initialize_test):
    """Test the new diameter roughness intervals Python GRPC interface"""

    # Create a test well
    well_path_coll = rips_instance.project.well_path_collection()
    well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    well_path.name = "Test Well for Diameter Intervals"

    completions_settings = well_path.completion_settings()
    assert completions_settings is not None, "Completion settings should not be None"

    # Test our new method - create first interval
    interval1 = completions_settings.add_diameter_roughness_interval(
        start_md=100, end_md=200, diameter=0.15, roughness_factor=1e-5
    )

    # Assertions to verify the first interval
    assert interval1 is not None, "First interval should not be None"
    assert interval1.start_md == 100
    assert interval1.end_md == 200
    assert interval1.diameter == 0.15
    assert interval1.roughness_factor == 1e-5

    # Test creating a second interval with different values
    interval2 = completions_settings.add_diameter_roughness_interval(
        start_md=200, end_md=300, diameter=0.12, roughness_factor=2e-5
    )

    # Assertions to verify the second interval
    assert interval2 is not None, "Second interval should not be None"
    assert interval2.start_md == 200
    assert interval2.end_md == 300
    assert interval2.diameter == 0.12
    assert interval2.roughness_factor == 2e-5

    # Test default values
    interval3 = completions_settings.add_diameter_roughness_interval()

    assert interval3 is not None, "Third interval with defaults should not be None"
    assert interval3.start_md == 0.0
    assert interval3.end_md == 100.0
    assert interval3.diameter == 0.152
    assert interval3.roughness_factor == 1e-5



import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import dataroot


def test_export_corner_point_grid_basic(rips_instance, initialize_test):
    """Test the export_corner_point_grid method with a simple created grid"""

    # Create a simple test case
    nx, ny, nz = 2, 2, 2

    # Generate simple coordinate data
    coord = []
    for j in range(ny + 1):
        for i in range(nx + 1):
            x = i * 100.0
            y = j * 100.0
            # Top point
            coord.extend([x, y, 1000.0])
            # Bottom point
            coord.extend([x, y, 1100.0])

    # Generate simple ZCORN data (8 values per cell)
    zcorn = []
    for k in range(nz):
        for j in range(ny):
            for i in range(nx):
                base_depth = 1000.0 + k * 100.0
                # 8 corner depths for each cell
                zcorn.extend(
                    [
                        base_depth,
                        base_depth,
                        base_depth,
                        base_depth,
                        base_depth + 100.0,
                        base_depth + 100.0,
                        base_depth + 100.0,
                        base_depth + 100.0,
                    ]
                )

    # Generate ACTNUM (all cells active)
    actnum = [1] * (nx * ny * nz)

    # Create the grid case
    case = rips_instance.project.create_corner_point_grid(
        "TestGrid", nx, ny, nz, coord, zcorn, actnum
    )

    # Test our export function
    exported_zcorn, exported_coord, exported_actnum, export_nx, export_ny, export_nz = (
        case.export_corner_point_grid()
    )

    # Basic validation
    assert len(exported_zcorn) == len(zcorn)
    assert len(exported_coord) == len(coord)
    assert len(exported_actnum) == len(actnum)

    # Check that all cells are active
    active_count = sum(1 for x in exported_actnum if x > 0)
    assert active_count == sum(actnum)

    # Verify dimensions are returned correctly
    assert export_nx == nx, f"Dimension mismatch: nx={export_nx} vs expected {nx}"
    assert export_ny == ny, f"Dimension mismatch: ny={export_ny} vs expected {ny}"
    assert export_nz == nz, f"Dimension mismatch: nz={export_nz} vs expected {nz}"


def test_export_corner_point_grid_return_types(rips_instance, initialize_test):
    """Test that export_corner_point_grid returns the correct types"""

    # Create a minimal test case
    nx, ny, nz = 1, 1, 1
    coord = [0, 0, 1000, 0, 0, 1100] * 4  # 4 coordinate lines for 1x1 grid
    zcorn = [1000] * 8  # 8 corners
    actnum = [1]  # 1 active cell

    case = rips_instance.project.create_corner_point_grid(
        "MinimalGrid", nx, ny, nz, coord, zcorn, actnum
    )

    exported_zcorn, exported_coord, exported_actnum, export_nx, export_ny, export_nz = (
        case.export_corner_point_grid()
    )

    # Check return types
    assert isinstance(exported_zcorn, list), "ZCORN should be a list"
    assert isinstance(exported_coord, list), "COORD should be a list"
    assert isinstance(exported_actnum, list), "ACTNUM should be a list"

    # Check element data types
    assert all(isinstance(x, float) for x in exported_zcorn)
    assert all(isinstance(x, float) for x in exported_coord)
    assert all(isinstance(x, int) for x in exported_actnum)

    # Check dimension types
    assert isinstance(export_nx, int), "nx dimension should be integer"
    assert isinstance(export_ny, int), "ny dimension should be integer"
    assert isinstance(export_nz, int), "nz dimension should be integer"


def test_export_corner_point_grid_roundtrip(rips_instance, initialize_test):
    """Test round-trip: load Eclipse grid, export data, recreate grid, and compare geometry"""

    # Load the Eclipse grid file (using BRUGGE case without LGR complications)
    original_case = rips_instance.project.load_case(
        dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
    )

    assert original_case is not None, "Failed to load test case"
    assert (
        original_case.name == "BRUGGE_0000"
    ), f"Expected case name BRUGGE_0000, got {original_case.name}"

    # Get original geometry data for comparison
    original_active_count = original_case.cell_count().active_cell_count
    original_cell_corners = original_case.active_cell_corners()

    # Export corner point grid data
    exported_zcorn, exported_coord, exported_actnum, nx, ny, nz = (
        original_case.export_corner_point_grid()
    )

    # Validate that the returned dimensions match the exported array sizes
    total_cells = len(exported_actnum)
    assert nx * ny * nz == total_cells

    # Validate array sizes make sense for these dimensions
    expected_coord_size = (nx + 1) * (ny + 1) * 6  # 6 values per coordinate line
    expected_zcorn_size = nx * ny * nz * 8  # 8 corner depths per cell
    expected_actnum_size = nx * ny * nz  # 1 value per cell

    assert len(exported_coord) == expected_coord_size
    assert len(exported_zcorn) == expected_zcorn_size
    assert len(exported_actnum) == expected_actnum_size

    # Create new corner point grid from exported data
    recreated_case = rips_instance.project.create_corner_point_grid(
        "Recreated_BRUGGE", nx, ny, nz, exported_coord, exported_zcorn, exported_actnum
    )

    assert recreated_case is not None, "Failed to create recreated case"

    # Compare geometry between original and recreated cases
    recreated_active_count = recreated_case.cell_count().active_cell_count
    recreated_cell_corners = recreated_case.active_cell_corners()

    # Compare active cell counts
    assert recreated_active_count == original_active_count

    # Compare number of corner points (should be same for active cells)
    assert len(recreated_cell_corners) == len(original_cell_corners)

    # Now do a focused comparison on a smaller sample
    sample_size = min(200, len(original_cell_corners))

    for i in range(sample_size):
        orig_cell = original_cell_corners[i]
        recreated_cell = recreated_cell_corners[i]

        for corner_idx in range(8):
            orig_corner = getattr(orig_cell, f"c{corner_idx}")
            recreated_corner = getattr(recreated_cell, f"c{corner_idx}")

            diff_x = abs(orig_corner.x - recreated_corner.x)
            diff_y = abs(orig_corner.y - recreated_corner.y)
            diff_z = abs(orig_corner.z - recreated_corner.z)

            max_diff = max(diff_x, diff_y, diff_z)
            assert max_diff < 1e-3



###########################################################################################
# This example will calculate the average value of the porosity property for a specified
# region in the reservoir model. The region is identified by its EQLNUM value.
###########################################################################################

import rips

resinsight = rips.Instance.find()

cases = resinsight.project.cases()

time_step = 0
region_number = 2

for case in cases:
    sum_poro = 0.0
    cell_cout = 0

    eqlnum = case.active_cell_property("STATIC_NATIVE", "EQLNUM", time_step)
    poro = case.active_cell_property("STATIC_NATIVE", "PORO", time_step)
    if len(eqlnum) != len(poro):
        print("Size of eqlnum and poro is not identical.")
        break

    for i in range(len(eqlnum)):
        if eqlnum[i] == region_number:
            sum_poro += poro[i]
            cell_cout += 1

    # Calculate the average porosity for the specified region
    if cell_cout > 0:
        average_poro = sum_poro / cell_cout
        print(
            f"Case {case.id}: Cell count {cell_cout} Average PORO for region {region_number} = {average_poro}"
        )



import uuid
import grpc

import SimulatorTables_pb2
import SimulatorTables_pb2_grpc

import PdmObject_pb2

from .pdmobject import add_method
from .project import Project
from .resinsight_classes import WellPath

from typing import Dict, List, Optional


@add_method(WellPath)
def __custom_init__(
    self: WellPath, pb2_object: PdmObject_pb2.PdmObject, channel: grpc.Channel
) -> None:
    self.__well_path_stub = SimulatorTables_pb2_grpc.WellPathStub(channel)


@add_method(WellPath)
def trajectory_properties(
    self: WellPath,
    resampling_interval: float,
) -> Dict[str, List[float]]:
    """Extracts properties from a well path trajectory.

    Arguments:
        resampling_interval (float): Interval in measured depth.

    Returns:
        Dict of properties.
    """

    if resampling_interval <= 0.0:
        raise ValueError("Resampling interval must be positive.")

    # Property names that will be used as both dict keys and temp key suffixes
    properties = [
        "coordinate_x",
        "coordinate_y",
        "coordinate_z",
        "measured_depth",
        "azimuth",
        "inclination",
        "dogleg",
    ]

    # Generate temporary keys with shared UUID
    shared_uuid = uuid.uuid4()
    temp_keys = {prop: f"{shared_uuid}_{prop}" for prop in properties}

    # Extract properties with temporary keys using **kwargs unpacking.
    # extract_well_path_properties puts results in the key-value store using
    # the specified keys.
    self.extract_well_path_properties_internal(
        resampling_interval=resampling_interval, **temp_keys
    )

    # Get the results from the key-value store.
    project = self.ancestor(Project)
    if project:
        result = {
            prop: project.key_values(temp_key) for prop, temp_key in temp_keys.items()
        }

        # Delete results form key-value store.
        for temp_key in temp_keys.values():
            project.remove_key_values(temp_key)

        return result
    else:
        return {}


@add_method(WellPath)
def add_well_log(
    self: WellPath,
    name: str,
    measured_depth: List[float],
    channel_data: Dict[str, List[float]],
    tvd_msl: Optional[List[float]] = None,
    tvd_rkb: Optional[List[float]] = None,
) -> object:
    """Add imported well log data to well path.

    Arguments:
        name (str): Name of the well log.
        measured_depth (List[float]): Array of measured depth values.
        channel_data (Dict[str, List[float]]): Dictionary mapping channel names to their values.
        tvd_msl (List[float], optional): Array of TVD MSL values. Must match measured_depth length if provided.
        tvd_rkb (List[float], optional): Array of TVD RKB values. Must match measured_depth length if provided.

    Returns:
        The created well log object.
    """

    if not name:
        raise ValueError("Name cannot be empty")

    if not measured_depth:
        raise ValueError("Measured depth list cannot be empty")

    if not isinstance(measured_depth, list):
        raise TypeError("Measured depth must be a list")

    if not channel_data:
        raise ValueError("Channel data dictionary cannot be empty")

    if not isinstance(channel_data, dict):
        raise TypeError("Channel data must be a dictionary")

    # Validate measured depth values are numeric
    try:
        float_measured_depth = [float(v) for v in measured_depth]
    except (ValueError, TypeError) as e:
        raise TypeError("All measured depth values must be numeric") from e

    # Validate optional TVD parameters
    validated_tvd_msl = None
    validated_tvd_rkb = None

    if tvd_msl is not None:
        if not isinstance(tvd_msl, list):
            raise TypeError("tvd_msl must be a list")

        if len(tvd_msl) != len(measured_depth):
            raise ValueError(
                f"tvd_msl has {len(tvd_msl)} values but measured depth has {len(measured_depth)} values"
            )

        try:
            validated_tvd_msl = [float(v) for v in tvd_msl]
        except (ValueError, TypeError) as e:
            raise TypeError("All tvd_msl values must be numeric") from e

    if tvd_rkb is not None:
        if not isinstance(tvd_rkb, list):
            raise TypeError("tvd_rkb must be a list")

        if len(tvd_rkb) != len(measured_depth):
            raise ValueError(
                f"tvd_rkb has {len(tvd_rkb)} values but measured depth has {len(measured_depth)} values"
            )

        try:
            validated_tvd_rkb = [float(v) for v in tvd_rkb]
        except (ValueError, TypeError) as e:
            raise TypeError("All tvd_rkb values must be numeric") from e

    # Validate channel data
    validated_channels = {}
    for channel_name, values in channel_data.items():
        if not channel_name:
            raise ValueError("Channel name cannot be empty")

        if not isinstance(values, list):
            raise TypeError(f"Values for channel '{channel_name}' must be a list")

        if not values:
            raise ValueError(
                f"Values list for channel '{channel_name}' cannot be empty"
            )

        if len(values) != len(measured_depth):
            raise ValueError(
                f"Channel '{channel_name}' has {len(values)} values but measured depth has {len(measured_depth)} values"
            )

        # Validate that all values are numeric
        try:
            validated_channels[channel_name] = [float(v) for v in values]
        except (ValueError, TypeError) as e:
            raise TypeError(
                f"All values in channel '{channel_name}' must be numeric"
            ) from e

    # Store data in key-value store and prepare for GRPC call
    project = self.ancestor(Project)
    if not project:
        raise RuntimeError("Could not find project")

    # Generate unique keys for data transfer
    shared_uuid = uuid.uuid4()
    measured_depth_key = f"{shared_uuid}_measured_depth"
    tvd_msl_key = f"{shared_uuid}_tvd_msl" if validated_tvd_msl is not None else ""
    tvd_rkb_key = f"{shared_uuid}_tvd_rkb" if validated_tvd_rkb is not None else ""

    # Store measured depth
    project.set_key_values(measured_depth_key, float_measured_depth)

    # Store optional TVD data
    if validated_tvd_msl is not None:
        project.set_key_values(tvd_msl_key, validated_tvd_msl)
    if validated_tvd_rkb is not None:
        project.set_key_values(tvd_rkb_key, validated_tvd_rkb)

    # Store all channels and build channel mappings
    temp_keys = [measured_depth_key]
    if validated_tvd_msl is not None:
        temp_keys.append(tvd_msl_key)
    if validated_tvd_rkb is not None:
        temp_keys.append(tvd_rkb_key)
    channel_mappings = []

    try:
        for channel_name, values in validated_channels.items():
            channel_key = f"{shared_uuid}_channel_{channel_name}"
            project.set_key_values(channel_key, values)
            temp_keys.append(channel_key)
            channel_mappings.append(f"{channel_name}:{channel_key}")

        # Create comma-separated string for channel mappings
        channel_keys_csv = ",".join(channel_mappings)

        # Call the internal GRPC method
        well_log = self.add_well_log_internal(
            name=name,
            measured_depth_key=measured_depth_key,
            channel_keys_csv=channel_keys_csv,
            tvd_msl_key=tvd_msl_key,
            tvd_rkb_key=tvd_rkb_key,
        )
        return well_log
    except Exception as e:
        # Clean up all temporary keys on failure
        for temp_key in temp_keys:
            try:
                project.remove_key_values(temp_key)
            except Exception:
                pass  # Ignore cleanup errors
        raise RuntimeError(f"Failed to create well log: {str(e)}") from e


@add_method(WellPath)
def completion_data(
    self: WellPath, case_id: int
) -> SimulatorTables_pb2.SimulatorTableData:
    """Get well completion data

    Returns a :ref:`simulatortabledata` object containing completion information
    for this well path.

    **SimulatorTableRequest description**::

       Parameter   | Description                                                   | Type
       ----------- | ------------------------------------------------------------- | -----
       well_name   | Well name as string                                           | string
       case_id     | ID of the case to use when extracting completion data         | int

    """
    sim_tab_req = SimulatorTables_pb2.SimulatorTableRequest(
        wellpath_name=self.name, case_id=case_id
    )
    return self.__well_path_stub.GetCompletionData(sim_tab_req)



import sys
import os
import pytest

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips


def test_set_and_get_values(rips_instance, initialize_test):
    project = rips_instance.project
    key = "my-key"
    input_values = [float(x) for x in range(2000)]
    project.set_key_values(key, input_values)

    output_values = project.key_values(key)
    assert input_values == output_values


def test_get_non_existing_key(rips_instance, initialize_test):
    project = rips_instance.project
    key = "this-key-does-not-exist"

    with pytest.raises(rips.RipsError, match="No matching key found."):
        project.key_values(key)


def test_remove_non_existing_key(rips_instance, initialize_test):
    project = rips_instance.project
    key = "this-key-does-not-exist"

    with pytest.raises(rips.RipsError, match="No matching key found."):
        project.remove_key_values(key)


def test_set_and_remove_and_get_values(rips_instance, initialize_test):
    project = rips_instance.project
    key = "my-key"
    input_values = [float(x) for x in range(2000)]
    project.set_key_values(key, input_values)

    output_values = project.key_values(key)
    assert input_values == output_values

    project.remove_key_values(key)

    with pytest.raises(rips.RipsError, match="No matching key found."):
        project.key_values(key)



import sys
import os
import pytest

sys.path.insert(1, os.path.join(sys.path[0], "../../"))


def test_import_well_path_from_points_basic(rips_instance, initialize_test):
    """Test basic creation of a fixed trajectory well path"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Define a simple vertical well path
    coordinates = [
        [1000.0, 2000.0, 0.0],  # Surface point
        [1000.0, 2000.0, -100.0],  # 100m down
        [1000.0, 2000.0, -500.0],  # 500m down
        [1000.0, 2000.0, -1000.0],  # 1000m down
    ]

    well_path = well_path_coll.import_well_path_from_points(
        name="TestFixedWell", coordinates=coordinates
    )

    assert well_path is not None
    assert well_path.name == "TestFixedWell"


def test_import_well_path_from_points_deviated(rips_instance, initialize_test):
    """Test creation of a deviated well path"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Define a deviated well path
    coordinates = [
        [1000.0, 2000.0, 0.0],  # Surface point
        [1020.0, 2010.0, -200.0],  # Slight deviation
        [1050.0, 2030.0, -500.0],  # More deviation
        [1100.0, 2080.0, -1000.0],  # Final target
    ]

    well_path = well_path_coll.import_well_path_from_points(
        name="TestDeviatedWell", coordinates=coordinates
    )

    assert well_path is not None
    assert well_path.name == "TestDeviatedWell"


def test_import_well_path_from_points_validation(rips_instance, initialize_test):
    """Test input validation for fixed trajectory well path"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Test empty name
    with pytest.raises(ValueError, match="Name cannot be empty"):
        well_path_coll.import_well_path_from_points(
            name="", coordinates=[[1000, 2000, 0]]
        )

    # Test empty coordinates
    with pytest.raises(ValueError, match="Coordinates list cannot be empty"):
        well_path_coll.import_well_path_from_points(name="TestWell", coordinates=[])

    # Test invalid coordinate format - wrong number of values
    with pytest.raises(ValueError, match="must be a list/tuple of 3 values"):
        well_path_coll.import_well_path_from_points(
            name="TestWell",
            coordinates=[[1000, 2000]],  # Missing Z coordinate
        )

    # Test invalid coordinate format - non-numeric values
    with pytest.raises(ValueError, match="All coordinate values must be numeric"):
        well_path_coll.import_well_path_from_points(
            name="TestWell", coordinates=[[1000, "invalid", 0]]
        )


def test_import_well_path_from_points_multiple(rips_instance, initialize_test):
    """Test creating multiple fixed trajectory well paths"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Create first well path
    coordinates1 = [
        [1000.0, 2000.0, 0.0],
        [1000.0, 2000.0, -500.0],
    ]

    well_path1 = well_path_coll.import_well_path_from_points(
        name="Well_1", coordinates=coordinates1
    )

    # Create second well path
    coordinates2 = [
        [1500.0, 2500.0, 0.0],
        [1500.0, 2500.0, -300.0],
    ]

    well_path2 = well_path_coll.import_well_path_from_points(
        name="Well_2", coordinates=coordinates2
    )

    assert well_path1.name == "Well_1"
    assert well_path2.name == "Well_2"
    assert well_path1 != well_path2


def test_import_well_path_from_points_single_point(rips_instance, initialize_test):
    """Test creating a well path with a single point"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Single point well path
    coordinates = [
        [1000.0, 2000.0, -100.0],
    ]

    well_path = well_path_coll.import_well_path_from_points(
        name="SinglePointWell", coordinates=coordinates
    )

    assert well_path is not None
    assert well_path.name == "SinglePointWell"


def test_import_well_path_from_points_various_types(rips_instance, initialize_test):
    """Test with various coordinate input types (list, tuple, mixed)"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Mix of list and tuple coordinates
    coordinates = [
        [1000.0, 2000.0, 0.0],  # List
        (1010.0, 2010.0, -100.0),  # Tuple
        [1020, 2020, -200],  # Integers
    ]

    well_path = well_path_coll.import_well_path_from_points(
        name="MixedTypesWell", coordinates=coordinates
    )

    assert well_path is not None
    assert well_path.name == "MixedTypesWell"


def test_import_well_path_from_points_large_coordinates(rips_instance, initialize_test):
    """Test with large coordinate values (typical UTM coordinates)"""
    well_path_coll = rips_instance.project.well_path_collection()

    # Large UTM-style coordinates
    coordinates = [
        [456789.0, 6789012.0, 50.0],  # Surface
        [456790.0, 6789013.0, -1500.0],  # Deep point
    ]

    well_path = well_path_coll.import_well_path_from_points(
        name="UTMCoordinatesWell", coordinates=coordinates
    )

    assert well_path is not None
    assert well_path.name == "UTMCoordinatesWell"



######################################################################
# This script creates a corner point grid from a Eclipse coord, zcorn
# and actnum configuration.
######################################################################
import rips
from xtgeo.grid3d._egrid import EGrid
from xtgeo.io._file import FileFormat
import numpy as np

grid_filepath = "/home/resinsight/testdata/01_drogon_ahm/realization-0/iter-0/eclipse/model/DROGON-0.EGRID"

name = "DROGON-0 from python"

grid = EGrid.from_file(grid_filepath, fileformat=FileFormat.EGRID)
print("Grid: ", grid)
print("Grid type: ", type(grid))
print("coord: ", grid.coord.shape, grid.coord.dtype)
print("zcorn:", grid.zcorn.shape, grid.zcorn.dtype)
print("actnum: ", grid.actnum.shape, grid.actnum.dtype)

resinsight = rips.Instance.find()

project = resinsight.project

coord = np.ascontiguousarray(grid.coord, dtype=np.float32)
zcorn = np.ascontiguousarray(grid.zcorn, dtype=np.float32)
actnum = np.ascontiguousarray(grid.actnum, dtype=np.int32)


print("coordsv: ", len(coord), type(coord[0]))
print("zcornsv: ", len(zcorn), type(zcorn[0]))
print("actnumsv: ", len(actnum), type(actnum[0]))


print("Grid dimensions: ", grid.dimensions)
nx = grid.dimensions.ncol
ny = grid.dimensions.nrow
nz = grid.dimensions.nlay

project.create_corner_point_grid(name, nx, ny, nz, coord, zcorn, actnum)



import sys
import os
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_append_lateral_from_measured_depth(rips_instance, initialize_test):
    well_path_coll = rips_instance.project.well_path_collection()

    # Create main well path using ModeledWellPath with targets
    main_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    main_well_path.name = "main_well_check_connection"
    main_well_path.update()

    # Add geometry targets to main well path
    main_geometry = main_well_path.well_path_geometry()
    main_geometry.append_well_target([1000.0, 2000.0, 0.0])  # Surface
    main_geometry.append_well_target([1000.0, 2000.0, -100.0])  # 100m down
    main_geometry.append_well_target([1000.0, 2000.0, -300.0])  # 300m down
    main_geometry.append_well_target([1000.0, 2000.0, -500.0])  # 500m down
    main_geometry.update()

    measured_depth = 150
    lateral = main_well_path.append_lateral(measured_depth)
    assert lateral is not None


def test_append_lateral(rips_instance, initialize_test):
    well_path_coll = rips_instance.project.well_path_collection()

    # Create main well path
    main_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    main_well_path.name = "main_well"
    main_well_path.update()

    # Create lateral well path
    lateral_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    lateral_well_path.name = "lateral_well"
    lateral_well_path.update()

    # Test appending lateral to main well path
    # This should fail because well paths don't have geometry defined
    try:
        main_well_path.append_lateral_from_geometry(well_path_lateral=lateral_well_path)
        # If we get here, the operation succeeded unexpectedly
        assert False, "Expected append_lateral to fail due to missing geometry"
    except rips.RipsError as e:
        # Expected error message when no geometry is available
        expected_msg = "No geometry available for main well path. Cannot append lateral to main well path."
        assert str(e) == expected_msg

    # Verify the lateral well path objects still exist and have correct names
    assert lateral_well_path is not None
    assert lateral_well_path.name == "lateral_well"
    assert main_well_path is not None
    assert main_well_path.name == "main_well"


def test_append_lateral_with_geometry(rips_instance, initialize_test):
    well_path_coll = rips_instance.project.well_path_collection()

    # Create main well path with geometry using coordinates
    main_coordinates = [
        [1000.0, 2000.0, 0.0],  # Surface point
        [1000.0, 2000.0, -100.0],  # 100m down
        [1000.0, 2000.0, -500.0],  # 500m down
        [1000.0, 2000.0, -1000.0],  # 1000m down
    ]

    main_well_path = well_path_coll.import_well_path_from_points(
        name="main_well_with_geometry", coordinates=main_coordinates
    )

    # Create lateral well path with geometry that starts along the main well path
    lateral_coordinates = [
        [1000.0, 2000.0, -300.0],  # Start along main well at 300m depth
        [1020.0, 2020.0, -320.0],  # Deviate slightly
        [1050.0, 2050.0, -350.0],  # Continue deviation
        [1100.0, 2100.0, -400.0],  # Final lateral point
    ]

    lateral_well_path = well_path_coll.import_well_path_from_points(
        name="lateral_well_with_geometry", coordinates=lateral_coordinates
    )

    # Test appending lateral to main well path - this should work with geometry
    # Note: The actual connection may still fail if the geometries don't align properly,
    # but we're testing that the function can be called with valid geometry objects
    try:
        main_well_path.append_lateral_from_geometry(well_path_lateral=lateral_well_path)
        # If successful, verify both well paths still exist
        assert main_well_path.name == "main_well_with_geometry"
        assert lateral_well_path.name == "lateral_well_with_geometry"
    except rips.RipsError as e:
        # If it fails, it should be for geometric reasons, not missing geometry
        error_msg = str(e)
        assert (
            "No geometry available" not in error_msg
        ), f"Unexpected geometry error: {error_msg}"
        # Other geometric errors (like alignment issues) are acceptable for this test
        print(f"Geometric alignment error (expected): {error_msg}")

    # Verify the well path objects are still valid regardless of connection result
    assert main_well_path is not None
    assert lateral_well_path is not None


def test_append_lateral_check_connection(rips_instance, initialize_test):
    well_path_coll = rips_instance.project.well_path_collection()

    # Create main well path using ModeledWellPath with targets
    main_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    main_well_path.name = "main_well_check_connection"
    main_well_path.update()

    # Add geometry targets to main well path
    main_geometry = main_well_path.well_path_geometry()
    main_geometry.append_well_target([1000.0, 2000.0, 0.0])  # Surface
    main_geometry.append_well_target([1000.0, 2000.0, -100.0])  # 100m down
    main_geometry.append_well_target([1000.0, 2000.0, -300.0])  # 300m down
    main_geometry.append_well_target([1000.0, 2000.0, -500.0])  # 500m down
    main_geometry.update()

    # Create lateral well path using ModeledWellPath with targets
    lateral_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    lateral_well_path.name = "lateral_well_check_connection"
    lateral_well_path.update()

    # Add geometry targets to lateral well path
    lateral_geometry = lateral_well_path.well_path_geometry()
    lateral_geometry.append_well_target(
        [1000.0, 2000.0, -150.0]
    )  # Start at 150m on main
    lateral_geometry.append_well_target([1020.0, 2020.0, -200.0])  # Deviate
    lateral_geometry.append_well_target([1050.0, 2050.0, -250.0])  # Continue
    lateral_geometry.update()

    # Get initial geometry state
    initial_attached_state = lateral_geometry.attached_to_parent_well

    # Test appending lateral to main well path
    try:
        main_well_path.append_lateral_from_geometry(well_path_lateral=lateral_well_path)

        # Check if connection was successful by examining the geometry properties
        lateral_geometry.update()  # Refresh from server
        final_attached_state = lateral_geometry.attached_to_parent_well

        # If connection was successful, verify the parent relationship
        if final_attached_state and not initial_attached_state:
            print("Successfully connected lateral to parent well path")
            assert lateral_well_path.name == "lateral_well_check_connection"
            assert main_well_path.name == "main_well_check_connection"

            # The lateral should now be marked as attached to parent
            assert lateral_geometry.attached_to_parent_well
        else:
            print("Connection did not change attachment state - may be geometric issue")
            # Still a valid test if the function completed without error

    except rips.RipsError as e:
        # Log the error for debugging but don't fail the test
        # Different geometric configurations may result in different connection outcomes
        error_msg = str(e)
        print(f"Connection attempt resulted in error: {error_msg}")

        # Ensure it's not a "no geometry" error since we provided geometry
        assert (
            "No geometry available" not in error_msg
        ), f"Unexpected geometry error: {error_msg}"

    # Verify both well paths are still valid
    assert main_well_path is not None
    assert lateral_well_path is not None
    assert main_well_path.name == "main_well_check_connection"
    assert lateral_well_path.name == "lateral_well_check_connection"



import pytest
import tempfile
import os


@pytest.fixture
def project(rips_instance):
    """Create a test project with a well path"""
    # Create well path from coordinates
    coordinates = [
        [1000, 2000, 0],
        [1000, 2000, 1000],
        [1100, 2100, 2000],
        [1200, 2200, 3000],
    ]

    well_path = (
        rips_instance.project.well_path_collection().import_well_path_from_points(
            name="TestWell", coordinates=coordinates
        )
    )

    return rips_instance.project, well_path


def test_well_log_project_file_persistence(project):
    """Test that well log data is persisted in project files"""
    proj, well_path = project

    # Test data with TVD values
    name = "PersistenceTestLog"
    measured_depth = [1000, 1500, 2000, 2500, 3000]
    channel_data = {
        "porosity": [0.10, 0.15, 0.20, 0.18, 0.12],
        "permeability": [10.5, 25.3, 45.8, 35.2, 18.7],
    }
    tvd_msl = [980, 1480, 1980, 2480, 2980]
    tvd_rkb = [1020, 1520, 2020, 2520, 3020]

    # Import well log
    well_log = well_path.add_well_log(
        name=name,
        measured_depth=measured_depth,
        channel_data=channel_data,
        tvd_msl=tvd_msl,
        tvd_rkb=tvd_rkb,
    )

    # Verify well log was created
    assert well_log is not None
    assert well_log.name == name

    # Save project to temporary file
    with tempfile.NamedTemporaryFile(suffix=".rsp", delete=False) as temp_file:
        temp_path = temp_file.name

    try:
        # Save the project
        proj.save(temp_path)

        # The project should save without errors, indicating PDM persistence is working
        # This test primarily verifies that the PDM objects can be serialized
        assert os.path.exists(temp_path)
        assert os.path.getsize(temp_path) > 0

        print(
            f"Project saved successfully to {temp_path} ({os.path.getsize(temp_path)} bytes)"
        )

    finally:
        # Clean up temp file
        if os.path.exists(temp_path):
            os.remove(temp_path)



######################################################################
# This script demonstrates how to export corner point grid data from
# ResInsight cases and recreate new grids from the exported data.
#
# This is the inverse operation of create_corner_point_grid.py - it
# extracts COORD, ZCORN, and ACTNUM arrays from existing Eclipse cases.
######################################################################

import rips


def validate_grid_dimensions(coord_array, zcorn_array, actnum_array, nx, ny, nz):
    """
    Validate that the exported arrays match the expected dimensions.

    Args:
        coord_array: COORD array (coordinate lines)
        zcorn_array: ZCORN array (corner depths)
        actnum_array: ACTNUM array (active cell flags)
        nx, ny, nz: Grid dimensions

    Returns:
        bool: True if arrays match expected dimensions
    """
    total_cells = nx * ny * nz
    expected_coord_size = (nx + 1) * (ny + 1) * 6  # 6 values per coordinate line
    expected_zcorn_size = nx * ny * nz * 8  # 8 corner depths per cell

    return (
        len(actnum_array) == total_cells
        and len(coord_array) == expected_coord_size
        and len(zcorn_array) == expected_zcorn_size
    )


def main():
    # Connect to ResInsight
    resinsight = rips.Instance.find()
    if resinsight is None:
        print("Starting ResInsight...")
        resinsight = rips.Instance.launch(console=True)

    project = resinsight.project
    if not project.cases():
        print("No cases in current project")
        return

    original_case = project.cases()[0]
    if original_case is None:
        print("Failed to load case")
        return

    print(f"Loaded case: {original_case.name}")

    # Get basic case information
    cell_count = original_case.cell_count()
    print("Case information:")
    print(f"  Total cells: {cell_count.reservoir_cell_count}")
    print(f"  Active cells: {cell_count.active_cell_count}")

    # Export corner point grid data
    print("\nExporting corner point grid data...")
    zcorn, coord, actnum, nx, ny, nz = original_case.export_corner_point_grid()

    print("Exported arrays:")
    print(f"  ZCORN: {len(zcorn):,} values (corner depths)")
    print(f"  COORD: {len(coord):,} values (coordinate lines)")
    print(f"  ACTNUM: {len(actnum):,} values (active cell flags)")

    # Grid dimensions are now returned directly from the export method!
    print(f"\nGrid dimensions from export: {nx} x {ny} x {nz} = {nx * ny * nz:,} cells")

    # Validate that the arrays match the expected dimensions
    if validate_grid_dimensions(coord, zcorn, actnum, nx, ny, nz):
        print("✓ Array sizes match expected dimensions perfectly")
    else:
        print("⚠ Array sizes don't match expected dimensions - this shouldn't happen")
        return

    # Create a new corner point grid from the exported data
    print("\nCreating new corner point grid from exported data...")
    recreated_name = f"{original_case.name}_Recreated"
    recreated_case = project.create_corner_point_grid(
        recreated_name, nx, ny, nz, coord, zcorn, actnum
    )

    if recreated_case is None:
        print("Failed to create recreated case")
        return

    print(f"Created recreated case: {recreated_case.name}")

    # Compare the two grids
    print("\nComparing original and recreated grids:")

    orig_cell_count = original_case.cell_count()
    recreated_cell_count = recreated_case.cell_count()

    print("Original grid:")
    print(f"  Total cells: {orig_cell_count.reservoir_cell_count:,}")
    print(f"  Active cells: {orig_cell_count.active_cell_count:,}")

    print("Recreated grid:")
    print(f"  Total cells: {recreated_cell_count.reservoir_cell_count:,}")
    print(f"  Active cells: {recreated_cell_count.active_cell_count:,}")

    # Get coordinate ranges for comparison
    print("\nCoordinate ranges:")

    # Sample some coordinates for comparison
    orig_bbox = original_case.reservoir_boundingbox()
    recreated_bbox = recreated_case.reservoir_boundingbox()

    print("Original bounding box:")
    print(f"  X: {orig_bbox.min_x:.2f} to {orig_bbox.max_x:.2f}")
    print(f"  Y: {orig_bbox.min_y:.2f} to {orig_bbox.max_y:.2f}")
    print(f"  Z: {orig_bbox.min_z:.2f} to {orig_bbox.max_z:.2f}")

    print("Recreated bounding box:")
    print(f"  X: {recreated_bbox.min_x:.2f} to {recreated_bbox.max_x:.2f}")
    print(f"  Y: {recreated_bbox.min_y:.2f} to {recreated_bbox.max_y:.2f}")
    print(f"  Z: {recreated_bbox.min_z:.2f} to {recreated_bbox.max_z:.2f}")


if __name__ == "__main__":
    main()



import pytest


@pytest.fixture
def project(rips_instance):
    """Create a test project with a well path"""
    # Create well path from coordinates
    coordinates = [
        [1000, 2000, 0],
        [1000, 2000, 1000],
        [1100, 2100, 2000],
        [1200, 2200, 3000],
    ]

    well_path = (
        rips_instance.project.well_path_collection().import_well_path_from_points(
            name="TestWell", coordinates=coordinates
        )
    )

    return rips_instance.project, well_path


def test_add_well_log_basic(project):
    """Test basic well log import functionality"""
    proj, well_path = project

    # Test data
    name = "TestLog"
    measured_depth = [1000, 1500, 2000, 2500, 3000]
    channel_data = {
        "porosity": [0.10, 0.15, 0.20, 0.18, 0.12],
        "permeability": [10.5, 25.3, 45.8, 35.2, 18.7],
    }

    # Import well log
    well_log = well_path.add_well_log(
        name=name, measured_depth=measured_depth, channel_data=channel_data
    )

    # Verify well log was created
    assert well_log is not None
    assert well_log.name == name


def test_add_well_log_empty_name(project):
    """Test that empty name raises ValueError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}

    with pytest.raises(ValueError, match="Name cannot be empty"):
        well_path.add_well_log(
            name="", measured_depth=measured_depth, channel_data=channel_data
        )


def test_add_well_log_empty_measured_depth(project):
    """Test that empty measured depth raises ValueError"""
    proj, well_path = project

    channel_data = {"porosity": [0.1, 0.2, 0.3]}

    with pytest.raises(ValueError, match="Measured depth list cannot be empty"):
        well_path.add_well_log(
            name="TestLog", measured_depth=[], channel_data=channel_data
        )


def test_add_well_log_empty_channel_data(project):
    """Test that empty channel data raises ValueError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]

    with pytest.raises(ValueError, match="Channel data dictionary cannot be empty"):
        well_path.add_well_log(
            name="TestLog", measured_depth=measured_depth, channel_data={}
        )


def test_add_well_log_invalid_measured_depth_type(project):
    """Test that non-list measured depth raises TypeError"""
    proj, well_path = project

    channel_data = {"porosity": [0.1, 0.2, 0.3]}

    with pytest.raises(TypeError, match="Measured depth must be a list"):
        well_path.add_well_log(
            name="TestLog", measured_depth="not a list", channel_data=channel_data
        )


def test_add_well_log_invalid_channel_data_type(project):
    """Test that non-dict channel data raises TypeError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]

    with pytest.raises(TypeError, match="Channel data must be a dictionary"):
        well_path.add_well_log(
            name="TestLog", measured_depth=measured_depth, channel_data="not a dict"
        )


def test_add_well_log_non_numeric_measured_depth(project):
    """Test that non-numeric measured depth values raise TypeError"""
    proj, well_path = project

    measured_depth = [1000, "invalid", 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}

    with pytest.raises(TypeError, match="All measured depth values must be numeric"):
        well_path.add_well_log(
            name="TestLog", measured_depth=measured_depth, channel_data=channel_data
        )


def test_add_well_log_non_numeric_channel_values(project):
    """Test that non-numeric channel values raise TypeError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, "invalid", 0.3]}

    with pytest.raises(
        TypeError, match="All values in channel 'porosity' must be numeric"
    ):
        well_path.add_well_log(
            name="TestLog", measured_depth=measured_depth, channel_data=channel_data
        )


def test_add_well_log_mismatched_lengths(project):
    """Test that mismatched lengths raise ValueError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2]}  # Only 2 values

    with pytest.raises(
        ValueError,
        match="Channel 'porosity' has 2 values but measured depth has 3 values",
    ):
        well_path.add_well_log(
            name="TestLog", measured_depth=measured_depth, channel_data=channel_data
        )


def test_add_well_log_empty_channel_name(project):
    """Test that empty channel name raises ValueError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"": [0.1, 0.2, 0.3]}  # Empty channel name

    with pytest.raises(ValueError, match="Channel name cannot be empty"):
        well_path.add_well_log(
            name="TestLog", measured_depth=measured_depth, channel_data=channel_data
        )


def test_add_well_log_mixed_numeric_types(project):
    """Test that mixed numeric types work correctly"""
    proj, well_path = project

    # Mix of int, float in both measured depth and channel data
    measured_depth = [1000, 1500.5, 2000, 2500.25, 3000]
    channel_data = {
        "porosity": [0.1, 0.15, 0.2, 0.18, 0.12],
        "gamma_ray": [75, 80.5, 85, 82.3, 78],
    }

    well_log = well_path.add_well_log(
        name="MixedLog", measured_depth=measured_depth, channel_data=channel_data
    )

    assert well_log is not None
    assert well_log.name == "MixedLog"


def test_add_well_log_multiple_channels(project):
    """Test adding well log with multiple channels"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000, 2500, 3000]
    channel_data = {
        "porosity": [0.10, 0.15, 0.20, 0.18, 0.12],
        "permeability": [10.5, 25.3, 45.8, 35.2, 18.7],
        "gamma_ray": [75.2, 82.1, 88.5, 85.3, 79.8],
        "resistivity": [2.5, 3.2, 1.8, 2.1, 2.8],
    }

    well_log = well_path.add_well_log(
        name="MultiChannelLog", measured_depth=measured_depth, channel_data=channel_data
    )

    assert well_log is not None
    assert well_log.name == "MultiChannelLog"


def test_add_multiple_well_logs(project):
    """Test adding multiple well logs to same well path"""
    proj, well_path = project

    # Add first well log
    measured_depth1 = [1000, 1500, 2000]
    channel_data1 = {"porosity": [0.1, 0.2, 0.3]}
    log1 = well_path.add_well_log(
        name="Log1", measured_depth=measured_depth1, channel_data=channel_data1
    )

    # Add second well log
    measured_depth2 = [1200, 1700, 2200, 2700]
    channel_data2 = {"gamma_ray": [75, 80, 85, 82]}
    log2 = well_path.add_well_log(
        name="Log2", measured_depth=measured_depth2, channel_data=channel_data2
    )

    # Verify both logs exist
    assert log1 is not None
    assert log2 is not None
    assert log1.name == "Log1"
    assert log2.name == "Log2"


def test_add_well_log_large_dataset(project):
    """Test importing large dataset"""
    proj, well_path = project

    # Generate large dataset
    measured_depth = [float(i * 10) for i in range(1000)]  # 0, 10, 20, ..., 9990
    channel_data = {
        "porosity": [0.1 + (i % 100) * 0.001 for i in range(1000)],
        "gamma_ray": [70.0 + (i % 50) * 0.5 for i in range(1000)],
    }

    well_log = well_path.add_well_log(
        name="LargeLog", measured_depth=measured_depth, channel_data=channel_data
    )

    assert well_log is not None
    assert well_log.name == "LargeLog"


def test_add_well_log_single_value(project):
    """Test adding well log with single value"""
    proj, well_path = project

    measured_depth = [1000.0]
    channel_data = {"porosity": [0.15]}

    well_log = well_path.add_well_log(
        name="SingleValue", measured_depth=measured_depth, channel_data=channel_data
    )

    assert well_log is not None
    assert well_log.name == "SingleValue"


def test_add_well_log_special_numeric_values(project):
    """Test adding well log with special numeric values"""
    proj, well_path = project

    measured_depth = [0.0, 1000.5, 2000.0, 3000.25, 4000.0]
    channel_data = {
        "special_values": [0.0, -5.5, 1e6, 1.23e-4, 99.999],
        "porosity": [0.0, 0.15, 0.25, 0.18, 0.0],
    }

    well_log = well_path.add_well_log(
        name="SpecialValues", measured_depth=measured_depth, channel_data=channel_data
    )

    assert well_log is not None
    assert well_log.name == "SpecialValues"


def test_add_well_log_with_tvd_msl(project):
    """Test adding well log with TVD MSL values"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000, 2500, 3000]
    channel_data = {"porosity": [0.10, 0.15, 0.20, 0.18, 0.12]}
    tvd_msl = [980, 1480, 1980, 2480, 2980]  # TVD MSL values

    well_log = well_path.add_well_log(
        name="WithTvdMsl",
        measured_depth=measured_depth,
        channel_data=channel_data,
        tvd_msl=tvd_msl,
    )

    assert well_log is not None
    assert well_log.name == "WithTvdMsl"


def test_add_well_log_with_tvd_rkb(project):
    """Test adding well log with TVD RKB values"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000, 2500, 3000]
    channel_data = {"porosity": [0.10, 0.15, 0.20, 0.18, 0.12]}
    tvd_rkb = [1020, 1520, 2020, 2520, 3020]  # TVD RKB values

    well_log = well_path.add_well_log(
        name="WithTvdRkb",
        measured_depth=measured_depth,
        channel_data=channel_data,
        tvd_rkb=tvd_rkb,
    )

    assert well_log is not None
    assert well_log.name == "WithTvdRkb"


def test_add_well_log_with_both_tvd(project):
    """Test adding well log with both TVD MSL and TVD RKB values"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000, 2500, 3000]
    channel_data = {"porosity": [0.10, 0.15, 0.20, 0.18, 0.12]}
    tvd_msl = [980, 1480, 1980, 2480, 2980]  # TVD MSL values
    tvd_rkb = [1020, 1520, 2020, 2520, 3020]  # TVD RKB values

    well_log = well_path.add_well_log(
        name="WithBothTvd",
        measured_depth=measured_depth,
        channel_data=channel_data,
        tvd_msl=tvd_msl,
        tvd_rkb=tvd_rkb,
    )

    assert well_log is not None
    assert well_log.name == "WithBothTvd"


def test_add_well_log_tvd_msl_length_mismatch(project):
    """Test that TVD MSL length mismatch raises ValueError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}
    tvd_msl = [980, 1480]  # Only 2 values for 3 measured depths

    with pytest.raises(
        ValueError, match="tvd_msl has 2 values but measured depth has 3 values"
    ):
        well_path.add_well_log(
            name="TestLog",
            measured_depth=measured_depth,
            channel_data=channel_data,
            tvd_msl=tvd_msl,
        )


def test_add_well_log_tvd_rkb_length_mismatch(project):
    """Test that TVD RKB length mismatch raises ValueError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}
    tvd_rkb = [1020, 1520, 2020, 2520]  # 4 values for 3 measured depths

    with pytest.raises(
        ValueError, match="tvd_rkb has 4 values but measured depth has 3 values"
    ):
        well_path.add_well_log(
            name="TestLog",
            measured_depth=measured_depth,
            channel_data=channel_data,
            tvd_rkb=tvd_rkb,
        )


def test_add_well_log_tvd_msl_non_numeric(project):
    """Test that non-numeric TVD MSL values raise TypeError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}
    tvd_msl = [980, "invalid", 1980]

    with pytest.raises(TypeError, match="All tvd_msl values must be numeric"):
        well_path.add_well_log(
            name="TestLog",
            measured_depth=measured_depth,
            channel_data=channel_data,
            tvd_msl=tvd_msl,
        )


def test_add_well_log_tvd_rkb_non_numeric(project):
    """Test that non-numeric TVD RKB values raise TypeError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}
    tvd_rkb = [1020, 1520, "invalid"]

    with pytest.raises(TypeError, match="All tvd_rkb values must be numeric"):
        well_path.add_well_log(
            name="TestLog",
            measured_depth=measured_depth,
            channel_data=channel_data,
            tvd_rkb=tvd_rkb,
        )


def test_add_well_log_tvd_msl_invalid_type(project):
    """Test that non-list TVD MSL raises TypeError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}
    tvd_msl = "not a list"

    with pytest.raises(TypeError, match="tvd_msl must be a list"):
        well_path.add_well_log(
            name="TestLog",
            measured_depth=measured_depth,
            channel_data=channel_data,
            tvd_msl=tvd_msl,
        )


def test_add_well_log_tvd_rkb_invalid_type(project):
    """Test that non-list TVD RKB raises TypeError"""
    proj, well_path = project

    measured_depth = [1000, 1500, 2000]
    channel_data = {"porosity": [0.1, 0.2, 0.3]}
    tvd_rkb = "not a list"

    with pytest.raises(TypeError, match="tvd_rkb must be a list"):
        well_path.add_well_log(
            name="TestLog",
            measured_depth=measured_depth,
            channel_data=channel_data,
            tvd_rkb=tvd_rkb,
        )
